/***************************************************************************/
/*                                                                         */
/*                                                                         */
/* SSPHOT A Photometry Package                                             */
/* ===========================                                             */
/*                                                                         */
/* This C-program is part of the SSPHOT photometry package.                */
/* See the links below for more information and documentation.             */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package and the documentation from the web-page:    */
/*                                                                         */
/* http://www.spaennare.se/ssphot.html                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

/*************************************************************/
/*  bdf2imf.c   SS June 96, Dec 97                           */
/*                                                           */
/*************************************************************/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "image.h"


int main(argc,argv) 
int argc;
char *argv[];
{
   struct image_head header; 
   short **image;

   FILE *infile;
   
   int i,j,k,n,xsize,ysize,hsize,nm,nmm;
  
   char *d;

   short *ss;
   short *sss;

   if (argc<2) {
      printf("Usage: %s image(I2).bdf image\n",argv[0]);
      exit(0);
   } /* if */


   if ((infile = fopen(argv[1],"rb")) == NULL) {
      printf("File not found.\n");
      exit(0);
   } /* if */

   hsize=6656/2;

   nm=hsize;

   ss=(short *)calloc(nm+1,sizeof(short));

   fread(&ss[1],sizeof(short),nm,infile);

   printf("%6d %6d %6d\n",ss[29],ss[31],ss[37]);

   xsize=ss[29];
   ysize=ss[31];
   hsize=ss[37];

   header.id=image_magic;
   header.typ=im_s16;
   header.off=512;
   header.hgh=ysize;
   header.wid=xsize;

   image = (short **)calloc(header.hgh,sizeof(short*));

   CreateIm_s16(image,header.hgh,header.wid);

   rewind(infile);

   nm=hsize/2;

   fread(&ss[1],sizeof(short),nm,infile);

   nmm=xsize*ysize;

   sss=(short *)calloc(nmm+1,sizeof(short));

   fread(&sss[1],sizeof(short),nmm,infile);

   k=1;

   for (i=ysize-1; i>=0; i--) {
      for (j=0; j<xsize; j++) {
          image[i][j]=sss[k];
          k++;
      } /* for j */
   } /* for i */


   WriteImage(argv[2],image,header,im_s16);

   fclose(infile);

} /* End  */
