/***************************************************************************/
/*                                                                         */
/*                                                                         */
/* SSPHOT A Photometry Package                                             */
/* ===========================                                             */
/*                                                                         */
/* This C-program is part of the SSPHOT photometry package.                */
/* See the links below for more information and documentation.             */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package and the documentation from the web-page:    */
/*                                                                         */
/* http://www.spaennare.se/ssphot.html                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

/*************************************************************/
/*   convf.c  SS Sept 93, Dec 97                             */
/*                                                           */
/*   Convolution using FFT                                   */
/*   Single precission.                                      */
/*************************************************************/

#include <math.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>


double timetic=(double)(CLOCKS_PER_SEC);

double pi=3.14159265358979323;


int main(argc,argv) 
int argc;
char *argv[];

{
   FILE *imfile,*kfile,*outfile;

   double size,invsize,eps,amp0,max,diffsec,sum,ftemp,ssize,ampr;

   int i,j,k,l,width,height,it,itmax,savenr,isave,temp;
   int kwidth,kheight,kworig,khorig,ndim,le,i2,j2,length,maxwh;
   int isign,tt;

   int ss[3];
   int xys[3];
   int kxys[3];

   float dd[3];

   char name[80];
   char outname[80];
   char psfname[80];
 
   clock_t time1,timediff;

   float *psf,*dc,*w;


if (argc < 4) {
   printf("Usage: %s inimage.fff outimage.fff psf.fff ampr\n",argv[0]);
   exit(0);
} /* if */

strcpy(name,argv[1]);
strcpy(outname,argv[2]);
strcpy(psfname,argv[3]);
ampr=atof(argv[4]);

if ((imfile = fopen(name,"rb")) == NULL) {
   printf("File not found.\n");
   exit(0);
} /* if */

if ((kfile = fopen(psfname,"rb")) == NULL) {
   printf("File not found.\n");
   exit(0);
} /* if */

outfile=fopen(outname,"wb");


fread(&xys[1],sizeof(int),2,imfile);

width=xys[1];
height=xys[2];

length=width*height;
le=length;

if (width > height) {
   maxwh=width;
}
else {
   maxwh=height;
} /* if */

dc = (float *)calloc(2*length+1,sizeof(float));
psf = (float *)calloc(2*length+1,sizeof(float));
w = (float *)calloc(2*maxwh+1,sizeof(float));


fread(&kxys[1],sizeof(int),2,kfile);
 
kwidth=kxys[1];
kheight=kxys[2];
kworig=(kwidth-1)/2;
khorig=(kheight-1)/2;

ndim=2;
ss[1]=height;
ss[2]=width;


size=(double)(le);
invsize=1.0/size;
eps=1.0;

for (i=1; i<=2*le; i=i+2) {
   psf[i]=0.0;
   psf[i+1]=0.0;
} /* for i */

sum=0.0;

for (i=0; i<kheight; i++) {
   for (j=0; j<kwidth; j++) {

      if ((i<kworig) && (j<khorig)) {
         i2=width-(kworig-i);
         j2=height-(khorig-j);
      }
      if ((i<kworig) && (j>=khorig)) {
         i2=width-(kworig-i);
         j2=j-khorig;
      }
      if ((i>=kworig) && (j<khorig)) {
         i2=i-kworig;
         j2=height-(khorig-j);
      }
      if ((i>=kworig) && (j>=khorig)) {
         i2=i-kworig;
         j2=j-khorig;
      }
     
      fread(&dd[1],sizeof(float),1,kfile);
      ftemp=dd[1];
      
      k=2*(i2*height+j2)+1;

      psf[k]=ftemp;
      psf[k+1]=0.0;

      sum=sum+ftemp;

   } /* for j */
} /* for i */

sum=1.0/sum;

for (i=1; i<=2*le; i=i+2) {
   psf[i]=sum*psf[i];
} /* for i */


k=1;
for (i=0; i<height; i++) {
   for (j=0; j<width; j++) {
      fread(&dd[1],sizeof(float),1,imfile);
      dc[k]=dd[1];
      dc[k+1]=0.0;
      k=k+2;
   } /* for j */
} /* for i */

time1=clock();


tt=0;
isign=-1;
dfourt_(&psf[1],&ss[1],&ndim,&isign,&tt,&w[1]);

dfourt_(&dc[1],&ss[1],&ndim,&isign,&tt,&w[1]);
 
cccmul(dc,dc,psf,le,invsize,1.0);

tt=1;
isign=1;

dfourt_(&dc[1],&ss[1],&ndim,&isign,&tt,&w[1]);

xys[1]=width;
xys[2]=height;

fwrite(&xys[1],sizeof(int),2,outfile);

k=1;
for (i=0; i<height; i++) {
   for (j=0; j<width; j++) {
      dd[1]=dc[k];
      fwrite(&dd[1],sizeof(float),1,outfile);
      k=k+2;
   } /* for j */
} /* for i */


fclose(imfile);
fclose(kfile);
fclose(outfile);


timediff=clock()-time1;

printf("%10.2f\n",(double)(timediff)/timetic);


} /* End */
