/***************************************************************************/
/*                                                                         */
/*                                                                         */
/* SSPHOT A Photometry Package                                             */
/* ===========================                                             */
/*                                                                         */
/* This C-program is part of the SSPHOT photometry package.                */
/* See the links below for more information and documentation.             */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package and the documentation from the web-page:    */
/*                                                                         */
/* http://www.spaennare.se/ssphot.html                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

/*************************************************************/
/*                                                           */
/*   findsmbf.c      By Stefan Spaennare                     */
/*   First version:  Dec 1995                                */
/*   Latest updates: Dec 1997, June 2002                     */
/*                                                           */
/*************************************************************/

#include <math.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <time.h>

double timetic=(double)(CLOCKS_PER_SEC);

double pi=3.14159265358979323;

double sgnr(x)
double x;
{
   if (x == 0.0) {
      return(0.0);
   }
   else {
      if (x >= 0.0) {
         return(1.0);
      }
      else {
         return(-1.0);
      } /* if */
   } /* if */
} /* sgnr */


void leastsquares(x0,y0,n,c1,c2)
double *x0;
double *y0;
int n;
double *c1;
double *c2;
{
   double nf,x,y,sx,sy,mx,my,xx,xy;
   double  sxx,sxy,cc2;
  
   int i;   
   
   sx=0.0;
   sy=0.0;
   xx=0.0;
   xy=0.0;

   for (i=1; i<=n; i++) {
      x=x0[i]; 
      y=y0[i]; 
      sx=sx+x;
      sy=sy+y;
      xx=xx+x*x;
      xy=xy+x*y;
   } /* for i */
   
   nf=1.0/(double)(n);

   mx=sx*nf;
   my=sy*nf; 
   sxx=xx-sx*sx*nf;
   sxy=xy-sx*sy*nf;

   cc2=sxy/sxx;
   *c1=my-cc2*mx;
   *c2=cc2;

} /* leastsquares */


void setmask(mask,xsize,ysize,xa,ya,na,r0,nsteps)
short **mask;
int xsize,ysize;
int **xa,**ya;
int *na;
double r0;
int nsteps;
{
   int i,j,k,xcc,ycc,xi,yi;
   int i2,j2,pk;

   double r,pi3,v,vs,xx,yy;
   double xs,ys,xss,yss,eps;
   double sin0,cos0;

   eps=0.0001;

   xcc=xsize/2;
   ycc=ysize/2;

   xx=(double)(xcc);
   yy=(double)(ycc);


for (i2=0; i2<nsteps; i2++) {
   for (j2=0; j2<nsteps; j2++) {

      xs=(double)(i2)/(double)(nsteps)+xx;
      ys=(double)(j2)/(double)(nsteps)+yy;

      pk=nsteps*i2+j2+1;

      for (i=0; i<xsize; i++) {
         for (j=0; j<ysize; j++) {
            mask[j][i]=0;
         } /* for j */
      } /* for i */

      pi3=4.0*pi+0.1*pi;

      xss=xs;
      yss=ys;
      r=0.0; 

      while ((r+0.5-eps) <= r0) {
         if (r < 0.5) {
	    vs=pi;
	 }
	 else {
	    vs=pi/(40.0*r);
	 } /* if */
         v=0.0;
         while (v <= pi3) {
            cos0=cos(v);
            sin0=sin(v);
            xi=(int)(r*cos0+xss+0.5-eps*sgnr(cos0));
            yi=(int)(r*sin0+yss+0.5-eps*sgnr(sin0));
            mask[yi][xi]=1;
            v=v+vs;
         } /* while */
         r=r+0.05;
      } /* while */

      k=1;
      for (i=0; i<xsize; i++) {
         for (j=0; j<ysize; j++) {
            if (mask[j][i] == 1) {
               xa[pk][k]=i-xcc;
               ya[pk][k]=j-ycc;
               k++;
            } /* if */
         } /* for j */
      } /* for i */

      na[pk]=k-1;

      printf("%6d",na[pk]);

   } /* for j2 */

   printf("\n");

} /* for i2 */

} /* setmask */ 


void setmoff(a,m,n,dx,dy,fwhm,beta) 
double **a;
int m,n;
double dx,dy;
double fwhm,beta;
{
   int i,j;

   double x,y,x0,y0,aa,cc,gamma,sum,temp;

   gamma=4.0*(exp(log(2.0)/beta)-1.0);

   aa=gamma/(fwhm*fwhm);
   cc=gamma/(fwhm*fwhm);

   x0=0.5*(double)(m+1)-dx;
   y0=0.5*(double)(n+1)-dy;

   sum=0.0;

   for (i=1; i<=m; i++) {
      for (j=1; j<=n; j++) {
     
         x=(double)(i)-x0;
         y=(double)(j)-y0;

         temp=exp(-beta*log(1.0+aa*x*x+cc*y*y));     

         a[i][j]=temp;

         sum=sum+temp;

      } /* for j */
   } /* for i */

   sum=1.0/sum;

   for (i=1; i<=m; i++) {
      for (j=1; j<=n; j++) {
         a[i][j]=sum*a[i][j];
      } /* for j */
   } /* for i */
      

} /* setmoff */


void interpsf(psf1,kw,kh,x,y,fwhm,beta)
double **psf1;
int kw,kh;
double x,y;
double fwhm,beta;
{
   int xi,yi;
   double dx,dy;

   xi=(int)(x);
   yi=(int)(y);
   dx=x-(double)(xi);
   dy=y-(double)(yi);
   dx=1.0-dx;
   dy=1.0-dy; 

   setmoff(psf1,kw,kh,dx,dy,fwhm,beta);
  
} /* interpsf */


void initpsf(psff,psf1,kw,kh,nsteps,fwhm,beta,xa,ya,na)
double **psff;
double **psf1;
int kw,kh;
int nsteps;
double fwhm,beta;
int **xa,**ya;
int *na;
{
   int k,i,j,k2,kw2,kh2,xi,yi,na2;

   double x,y;

   kw2=kw/2;
   kh2=kh/2; 

   for (i=0; i<nsteps; i++) {
      for (j=0; j<nsteps; j++) {

         x=(double)(i)/(double)(nsteps);
         y=(double)(j)/(double)(nsteps);

         interpsf(psf1,kw,kh,x,y,fwhm,beta);

         k2=nsteps*i+j+1;

         na2=na[k2];

         for (k=1; k<=na2; k++) {

            xi=xa[k2][k]+kw2;
            yi=ya[k2][k]+kh2;

            psff[k2][k]=psf1[xi][yi];

         } /* for k */

      } /* for j */

    } /* for i */

} /* initpsf */


void getimage(image,imf,xa,ya,na,xc,yc)
float **image;
double *imf;
int *xa,*ya;
int na;
int xc,yc;
{
   int k,xi,yi; 

   for (k=1; k<=na; k++) {

      xi=xa[k]+xc;
      yi=ya[k]+yc;

      imf[k]=(double)(image[yi][xi]);

   } /* for k */

} /* getimage */ 


double getsigma(psff1,imf,na,c1,c2)
double *psff1;
double *imf;
int na;
double c1;
double c2;
{
   int k;

   double temp,ssum,sigma;

   ssum=0.0;

   for (k=1; k<=na; k++) {

      temp=c2*psff1[k]+c1-imf[k];

      ssum=ssum+temp*temp;

   } /* for k */

   sigma=sqrt(ssum/(double)(na-1));

   return(sigma);

} /* getssum */ 



int main(argc,argv) 
int argc;
char *argv[];

{

   FILE *outfile,*infile;

   int i,j,nsteps;
   int xsize,ysize,xsize2,ysize2; 
   int kh,kw,kw1,kh1;
   int maa,rri;
   int off,nsteps2,pk,px,py,off2;
   int ppx,ppy,mxsize,mysize;

   double amp1,minint,sigmafac,sigma,nstepsinv;
   double fwhm,beta,inte;
   double rr,gamma,alfa,eps;
   double xs,ys,r0,bg;
   double c1,c2;

   float data[6];

   clock_t time1,timediff1;

   double **psf1;
   double *mpsf1;


   double **psff;
   double *mppsff;


   short **mask;
   short *mpmask;

   double *imf;

   float **image;
   float *mpimage;

   float fff[3];

   int xys[3];

   int  **xa,**ya;
   int *mpxa,*mpya;

   int *na;



if (argc != 10) {
   printf("Usage: %s image.fff fwhmr betar rfitr minintr sigmafacr\n",argv[0]); 
   printf("       ampr nstepsi outfile.bin\n");
   exit(0);
} /* if */


fwhm=atof(argv[2]);
beta=atof(argv[3]);
r0=atof(argv[4]);
minint=atof(argv[5]);
sigmafac=atof(argv[6]);
amp1=atof(argv[7]);
nsteps=atoi(argv[8]);


if (beta < 1.8) {
   printf("Warning: beta < 1.8, program terminated !\n");
   exit(0);
} /* if */


if ((infile = fopen(argv[1],"rb")) == NULL) {
   printf("File %s not found.\n",argv[1]);
   exit(0);
} /* if */


outfile=fopen(argv[9],"wb");


eps=0.001;

gamma=4.0*(exp(log(2.0)/beta)-1.0);
alfa=sqrt((exp(log(eps)/(1.0-beta))-1.0)/gamma);

rr=fwhm*alfa;

rri=(int)(rr+1.5);

kh=2*rri+1;

kw=kh;

kw1=kw+1;
kh1=kh+1;

printf("%6d\n",kh);

fread(&xys[1],sizeof(int),2,infile);

xsize=xys[1];
ysize=xys[2];

image=(float **)calloc(ysize,sizeof(float*));
mpimage=(float *)calloc(xsize*ysize,sizeof(float));

for (i=0; i<ysize; i++) {
   image[i]=mpimage;
   mpimage=mpimage+xsize;
} /* for i */

for (i=0; i<ysize; i++) {
   for (j=0; j<xsize; j++) {
      fread(&fff[1],sizeof(float),1,infile);
      image[i][j]=fff[1];
   } /* for j */
} /* for i */   

fclose(infile);

mxsize=2*(int)(r0+1.0)+10;
mysize=2*(int)(r0+1.0)+10;

mask=(short **)calloc(mysize,sizeof(short*));
mpmask=(short *)calloc(mxsize*mysize,sizeof(short));

for (i=0; i<mysize; i++) {
   mask[i]=mpmask;
   mpmask=mpmask+mxsize;
} /* for i */


psf1=(double **)calloc(kw1+1,sizeof(double*));
mpsf1=(double *)calloc((kw1+1)*(kh1+1),sizeof(double));

for (i=0; i<=kw1; i++) {
   psf1[i]=mpsf1;
   mpsf1=mpsf1+kh1+1;
} /* for i */


nsteps2=nsteps*nsteps;

maa=(int)(pi*(r0+1.0)*(r0+1.0)+0.5);

xa=(int **)calloc(nsteps2+1,sizeof(int*));
ya=(int **)calloc(nsteps2+1,sizeof(int*));

mpxa=(int *)calloc((nsteps2+1)*(maa+1),sizeof(int));
mpya=(int *)calloc((nsteps2+1)*(maa+1),sizeof(int));

for (i=0; i<=nsteps2; i++) {
    xa[i]=mpxa;
    ya[i]=mpya;
    mpxa=mpxa+maa+1;
    mpya=mpya+maa+1;
} /* for i */

na=(int *)calloc(nsteps2+1,sizeof(int));


psff=(double **)calloc(nsteps2+1,sizeof(double*));

mppsff=(double *)calloc((nsteps2+1)*(maa+1),sizeof(double));

for (i=0; i<=nsteps2; i++) {
    psff[i]=mppsff;
    mppsff=mppsff+maa+1;
} /* for i */


imf=(double *)calloc(maa+1,sizeof(double));


time1=clock();

setmask(mask,mxsize,mysize,xa,ya,na,r0,nsteps);

initpsf(psff,psf1,kw,kh,nsteps,fwhm,beta,xa,ya,na);

off=(int)(r0+2.5);

off2=off*nsteps;

xsize2=xsize*nsteps-off2;
ysize2=ysize*nsteps-off2;

nstepsinv=1.0/(double)(nsteps);


for (i=off2; i<xsize2; i++) {
    for (j=off2; j<ysize2; j++) {

       ppx=i / nsteps;
       ppy=j / nsteps;
       px=i % nsteps;
       py=j % nsteps;

       pk=nsteps*px+py+1;

       getimage(image,imf,xa[pk],ya[pk],na[pk],ppx,ppy);

       leastsquares(psff[pk],imf,na[pk],&c1,&c2);

       inte=amp1*c2;
       
       if (inte > minint) {

          sigma=amp1*getsigma(psff[pk],imf,na[pk],c1,c2);

          bg=amp1*c1;

          if (sigma < (sigmafac*inte)) {
                
             xs=nstepsinv*(double)(i);
             ys=nstepsinv*(double)(j);

             data[1]=(float)(xs);
             data[2]=(float)(ys);
             data[3]=(float)(inte);
             data[4]=(float)(bg);
             data[5]=(float)(sigma);

             fwrite(&data[1],sizeof(float),5,outfile);

          } /* if */

       } /* if */

   } /* for j */
} /* for i */

timediff1=clock()-time1;

printf("%9.2f\n",(double)(timediff1)/timetic);

fclose(outfile);

} /* End  */
