/***************************************************************************/
/*                                                                         */
/*                                                                         */
/* SSPHOT A Photometry Package                                             */
/* ===========================                                             */
/*                                                                         */
/* This C-program is part of the SSPHOT photometry package.                */
/* See the links below for more information and documentation.             */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package and the documentation from the web-page:    */
/*                                                                         */
/* http://www.spaennare.se/ssphot.html                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

/*************************************************************/
/*  group.c                                                 */
/*                                                           */
/* Latest update: Mar 94 SS                                  */
/*************************************************************/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>


void qqsort(a,b,c,d,sr,sl,m,n)
double *a;
double *b;
double *c;
double *d;
int *sr;
int *sl;
int m;
int n;
{
  int l,r,s,i,j;

  double d2,temp;

  s=1;
  sl[s]=m;
  sr[s]=n;

  do {
     l=sl[s];
     r=sr[s];
     s--;
    do { 
     i=l;
     j=r;
     d2=d[(l+r)/2];
     do {
        while (d[i] < d2) {
           i++;
        }
        while (d[j] > d2) {
           j--;
        }
        if (i <= j) {
           temp=a[i];
           a[i]=a[j];
           a[j]=temp;
           temp=b[i];
           b[i]=b[j];
           b[j]=temp;
           temp=c[i];
           c[i]=c[j];
           c[j]=temp;
           temp=d[i];
           d[i]=d[j];
           d[j]=temp;
           i++;
           j--;
        }
     } while (i<=j);
     if (i<r) {
        s++;
        sl[s]=i;
        sr[s]=r;
     }
     r=j;
    } while (l<r);
  } while (s > 0);
   
} /* qqsort */



int main(argc,argv) 
int argc;
char *argv[];

{
   FILE *infile;

   int n,i,j,i2,i3,i4,mg,ls;
   double rmax,r,x,y,dx,dy,int2;

   double *x1,*y1,*int1,*r1;
  
   int *sr,*sl; 

   if (argc != 4) {
      printf("Usage: %s file(x y int) maxgroupi rmaxr\n",argv[0]);
      exit(0);
   } /* if */ 


   mg=atoi(argv[2]);
   rmax=atof(argv[3]);


   if ((infile = fopen(argv[1],"r")) == NULL) {
      printf("File %s not found.\n",argv[1]);
      exit(0);
   } /* if */

   i=1;

   while (fscanf(infile,"%lf %lf %lf",&x,&y,&int2) != EOF) {
      i++;
   } /* while */

   n=i-1;

   ls=(int)(2.0*log((double)(n))/log(2.0)+0.5);

   x1=(double *)calloc(n+2,sizeof(double));
   y1=(double *)calloc(n+2,sizeof(double));
   int1=(double *)calloc(n+2,sizeof(double));
   r1=(double *)calloc(n+2,sizeof(double));

   sr=(int *)calloc(ls+1,sizeof(int));
   sl=(int *)calloc(ls+1,sizeof(int));


   rewind(infile);

   i=1;

   while (fscanf(infile,"%lf %lf %lf",&x,&y,&int2) != EOF) {
      x1[i]=x;
      y1[i]=y;
      int1[i]=int2;
      i++;
   } /* while */


   rmax=rmax*rmax;


   i=1;
   i2=1;
   i3=1;
   i4=i;

   while (i<n) {
 
      x=x1[i];
      y=y1[i];

      i2++;

      for (j=i2; j<=n; j++) {
         dx=x-x1[j];
         dy=y-y1[j];
         r=dx*dx+dy*dy;
         r1[j]=r;
      } /* for j */

      qqsort(x1,y1,int1,r1,sr,sl,i2,n);


      while ((r1[i2] <= rmax) && (i3 < mg) && (i2 <= n)) {
         i2++;
         i3++;
      } /* while */
 
      i2--; 

      if ((i == i2) || (i3 == mg) || (i2 == n)) {
          for (j=i4; j<=i2; j++) {
             printf("%10.3f %10.3f %15.4f\n",x1[j],y1[j],int1[j]);
          } /* for j */
          printf("%10d %10d %15d\n",-1,-1,-1);
          i2++;
          i=i2;
          i3=1;
          i4=i;
      } 
      else {
          i++;
      } /* if */
   
   } /* while */

   if (i2 == n) {
      j=n;
      printf("%10.3f %10.3f %15.4f\n",x1[j],y1[j],int1[j]);
      printf("%10d %10d %15d\n",-1,-1,-1);
   } /* if */
    
   printf("%10d %10d %15d\n",-100,-100,-100);

                
} /* End */
