/***************************************************************************/
/*                                                                         */
/*                                                                         */
/* SSPHOT A Photometry Package                                             */
/* ===========================                                             */
/*                                                                         */
/* This C-program is part of the SSPHOT photometry package.                */
/* See the links below for more information and documentation.             */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package and the documentation from the web-page:    */
/*                                                                         */
/* http://www.spaennare.se/ssphot.html                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

/*************************************************************/
/*  imcombf.c   SS June 98                                   */
/*                                                           */
/*************************************************************/

#include <math.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>


int main(argc,argv) 
int argc;
char *argv[];
{
   FILE *infile[11][11];
   FILE *outfile;

   int xys[3];
   int xys2[3];

   float fff[3];
   
   int i,j,k,i2,j2;
   int nrow,ncol,xsize,ysize;
   int xsize2,ysize2;

   float **image[11][11];
   float **image2;
   float *mpimage[11][11];
   float *mpimage2;

   if (argc<5) {
      printf("Usage: %s nrowi ncoli\n",argv[0]);
      printf("            im_11.fff  im_12.fff  ... im_1nc.fff\n");
      printf("            im_21.fff  im_22.fff  ... im_2nc.fff\n");
      printf("                ...       ...     ...     ...\n");
      printf("            im_nr1.fff im_nr2.fff ... im_nrnc.fff\n");
      printf("            outimage.fff\n");
      exit(0);
   } /* if */

   nrow=atoi(argv[1]);
   ncol=atoi(argv[2]);

 
for (i2=1; i2<=nrow; i2++) {
   for (j2=1; j2<=ncol; j2++) {

   if ((infile[i2][j2] = fopen(argv[ncol*(i2-1)+j2+2],"rb")) == NULL) {
      printf("File not found.\n");
      exit(0);
   } /* if */

   fread(&xys[1],sizeof(int),2,infile[i2][j2]);

   xsize=xys[1];
   ysize=xys[2];

   image[i2][j2]=(float **)calloc(ysize,sizeof(float*));
   mpimage[i2][j2]=(float *)calloc(xsize*ysize,sizeof(float));

   for (i=0; i<ysize; i++) {
      image[i2][j2][i]=mpimage[i2][j2];
      mpimage[i2][j2]=mpimage[i2][j2]+xsize;
   } /* for i */

   for (i=0; i<ysize; i++) {
      for (j=0; j<xsize; j++) {
          fread(&fff[1],sizeof(float),1,infile[i2][j2]);
	  image[i2][j2][i][j]=fff[1];
      } /* for j */
   } /* for i */

   fclose(infile[i2][j2]);

   } /* for j2 */
} /* for i2 */


   xys2[1]=ncol*xys[1];
   xys2[2]=nrow*xys[2];

   xsize2=xys2[1];
   ysize2=xys2[2];

   image2=(float **)calloc(ysize2,sizeof(float*));
   mpimage2=(float *)calloc(xsize2*ysize2,sizeof(float));

   for (i=0; i<ysize2; i++) {
      image2[i]=mpimage2;
      mpimage2=mpimage2+xsize2;
   } /* for i */

   for (i2=0; i2<nrow; i2++) {
      for (j2=0; j2<ncol; j2++) {
         for (i=0; i<xsize; i++) {
            for (j=0; j<ysize; j++) {
                image2[j+i2*ysize][i+j2*xsize]=image[i2+1][j2+1][j][i];
            } /* for j */
         } /* for i */
      } /* for j2 */
   } /* for i2 */

   outfile=fopen(argv[ncol*nrow+3],"wb");

   fwrite(&xys2[1],sizeof(int),2,outfile);

   for (i=0; i<ysize2; i++) {
      for (j=0; j<xsize2; j++) {
         fff[1]=image2[i][j];
         fwrite(&fff[1],sizeof(float),1,outfile);
      } /* for j */
   } /* for i */

   fclose(outfile);


} /* End  */
