/***************************************************************************/
/*                                                                         */
/*                                                                         */
/* SSPHOT A Photometry Package                                             */
/* ===========================                                             */
/*                                                                         */
/* This C-program is part of the SSPHOT photometry package.                */
/* See the links below for more information and documentation.             */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package and the documentation from the web-page:    */
/*                                                                         */
/* http://www.spaennare.se/ssphot.html                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

/*************************************************************/
/*  imimadd.c   SS Dec 93                                    */
/*                                                           */
/*************************************************************/

#include <math.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include "image.h"


int main(argc,argv) 
int argc;
char *argv[];
{
   struct image_head header,header2; 
   short **image,**image2;
   
   int i,j,temp;

   double amp1,amp2;

   if (argc<5) {
      printf("Usage: %s inimage1 inimage2 outimage amp1r amp2r\n",argv[0]);
      exit(0);
   } /* if */

   amp1=atof(argv[4]);
   amp2=atof(argv[5]);

 
   ReadHeader(argv[1],&header);

   image = (short **)calloc(header.hgh,sizeof(short*));

   ReadImage(argv[1],image);


   ReadHeader(argv[2],&header2);

   image2 = (short **)calloc(header2.hgh,sizeof(short*));

   ReadImage(argv[2],image2);


   for (i=0; i<header.hgh; i++) {
      for (j=0; j<header.wid; j++) {
         temp=(int)(amp1*(double)(image[i][j])+amp2*(double)(image2[i][j])+0.5);
	 if (temp < 0) {
	    temp=0;
	 } /* if */
	 if (temp > 32767) {
	    temp=32767;
	 } /* if */
	 image2[i][j]=temp;
      } /* for j */
   } /* for i */


   WriteImage(argv[3],image2,header2,im_s16);

} /* End  */
