/***************************************************************************/
/*                                                                         */
/*                                                                         */
/* SSPHOT A Photometry Package                                             */
/* ===========================                                             */
/*                                                                         */
/* This C-program is part of the SSPHOT photometry package.                */
/* See the links below for more information and documentation.             */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package and the documentation from the web-page:    */
/*                                                                         */
/* http://www.spaennare.se/ssphot.html                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

/*************************************************************/
/*  imsig.c   SS Jan 96                                      */
/*                                                           */
/*************************************************************/

#include <math.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include "image.h"


void qqsort(d,m,n)
double *d;
int m;
int n;
{
  int l,r,s,i,j;

  double d2,temp;

  int sr[65];
  int sl[65];

  s=1;
  sl[s]=m;
  sr[s]=n;

  do {
     l=sl[s];
     r=sr[s];
     s--;
    do { 
     i=l;
     j=r;
     d2=d[(l+r)/2];
     do {
        while (d[i] < d2) {
           i++;
        } /* while */
        while (d[j] > d2) {
           j--;
        } /* while */
        if (i <= j) {
           temp=d[i];
           d[i]=d[j];
           d[j]=temp;
           i++;
           j--;
        } /* if */
     } while (i <= j);
     if (i < r) {
        s++;
        sl[s]=i;
        sr[s]=r;
     } /* if */
     r=j;
    } while (l < r);
  } while (s > 0);
   
} /* qqsort */


void quantiles(x,n,q,nq)
double *x;
int n;
double *q;
int nq;
{
   int iq,i,i0;
   double p,x0,x1;

   if (n == 1) {
      for (iq=1; iq<=(nq-1); iq++) {
         q[iq]=x[1];
      } /* for iq */
   }
   else {

      qqsort(x,1,n);

      for (i=1; i<=(nq-1); i++) {

         p=(double)(n)*(double)(i)/(double)(nq);
         i0=(int)(p);

         if  (i0 == 0) {
            x0=x[1];
            x1=0.5*(x[1]+x[2]);
         }
         else if (i0 == (n-1)) {
            x0=0.5*(x[n-1]+x[n]);
            x1=x[n];
         }
         else {
            x0=0.5*(x[i0]+x[i0+1]);
            x1=0.5*(x[i0+1]+x[i0+2]);
         } /* if */

         q[i]=x0+(x1-x0)*(p-(double)(i0));

      } /* for i */

   } /* if */

} /* quantiles */
             

void rob_ave_std(x,n,ave,std)
double *x;
int n;
double *ave;
double *std;
{
   double q[7];

   if (n == 0) {

      *ave=0.0;
      *std=0.0;

   }
   else {

      quantiles(x,n,q,6);

      *ave=q[3];
      *std=(q[5]-q[1])/1.935;

   } /* if */

} /* rob_ave_std */


void ave_std(x,n,ave,std)
double *x;
int n;
double *ave;
double *std;
{

   int i;

   double mean,sigma,ssum,temp;

   if (n == 0) {

      *ave=0.0;
      *std=0.0;

   } /* if */

   if (n == 1) {

      *ave=x[1];
      *std=0.0;

   } /* if */

   if (n > 1) {

      mean=0.0;

      for (i=1; i<=n; i++) {
         mean=mean+x[i];
      } /* if */

      mean=mean/(double)(n);

      ssum=0.0;

      for (i=1; i<=n; i++) {
         temp=x[i]-mean;
         ssum=ssum+temp*temp;
      } /* if */

      sigma=sqrt(ssum/(double)(n-1));

      *ave=mean;
      *std=sigma;

   } /* if */

} /* ave_std */



int main(argc,argv) 
int argc;
char *argv[];
{
   struct image_head header; 
   short **image;

   int i,j,xsize,ysize,n,k;

   double mean,sigma,rmean,rsigma;

   double *v;


   if (argc != 1) {
      printf("Usage: %s image \n",argv[0]);
      exit(0);
   } /* if */

   ReadHeader(argv[1],&header);

   image = (short **)calloc(header.hgh,sizeof(long));

   ReadImage(argv[1],image);

   xsize=header.wid;
   ysize=header.hgh;

   n=xsize*ysize;

   v=(double *)calloc(n+1,sizeof(double));

   k=1;
   for (i=0; i<xsize; i++) {
       for (j=0; j<ysize; j++) {
          v[k]=(double)(image[j][i]);
          k++;
       } /* for j */
   } /* for i */


   ave_std(v,n,&mean,&sigma);

   rob_ave_std(v,n,&rmean,&rsigma);

   printf("xsize ysize      npix       mean      sigma   rob_mean  rob_sigma\n");

   printf("%5d %5d %9d %10.3f %10.3f %10.3f %10.3f\n",
          xsize,ysize,n,mean,sigma,rmean,rsigma);


} /* End  */
