/***************************************************************************/
/*                                                                         */
/*                                                                         */
/* SSPHOT A Photometry Package                                             */
/* ===========================                                             */
/*                                                                         */
/* This C-program is part of the SSPHOT photometry package.                */
/* See the links below for more information and documentation.             */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package and the documentation from the web-page:    */
/*                                                                         */
/* http://www.spaennare.se/ssphot.html                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

/*************************************************************/
/*   imstarsmf.c    SS June 96, Dec 97                       */
/*                                                           */
/*************************************************************/

#include <math.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>

double timetic=(double)(CLOCKS_PER_SEC);

double pi=3.14159265358979323;


void setmoff(a,b,m,n,dx,dy,fwhm,beta) 
double **a;
double **b;
int m,n;
double dx,dy;
double fwhm,beta;
{
   int i,j;

   double x,y,x0,y0,aa,cc,gamma,sum,temp;

   gamma=4.0*(exp(log(2.0)/beta)-1.0);

   aa=gamma/(fwhm*fwhm);
   cc=gamma/(fwhm*fwhm);

   x0=0.5*(double)(m+1)-dx;
   y0=0.5*(double)(n+1)-dy;

   sum=0.0;

   for (i=1; i<=m; i++) {
      for (j=1; j<=n; j++) {
     
         x=(double)(i)-x0;
         y=(double)(j)-y0;

         temp=exp(-beta*log(1.0+aa*x*x+cc*y*y));     

         a[i][j]=temp;

         sum=sum+temp;

      } /* for j */
   } /* for i */

   sum=1.0/sum;

   for (i=1; i<=m; i++) {
      for (j=1; j<=n; j++) {
         b[i][j]=sum*a[i][j];
      } /* for j */
   } /* for i */
      

} /* setmoff */




int main(argc,argv) 
int argc;
char *argv[];

{

   FILE *infile,*imfile;

   double amp0,max,diffsec,sum,ftemp,ampr;
   double x,y,inte,dx,dy,rmax2,maxlev;

   double fwhm,beta,eps,gamma,alfa,rr;

   float fftemp;

   int i,j,k,l,k2;
   int xsize,ysize; 
   int xi,yi,i2,j2;

   int kh,kw,kw1,kh1,kw2,kh2;
   int rri; 

   int xys[3];

   char name[80];
   char psfname[80];
   char filename[80];


   double **psf1,**psf2;

   double *mpsf1,*mpsf2;

   float **fimage;

   float *mpfimage;

   float dd[3];
 
   clock_t time1,timediff;



if (argc < 7) {
   printf("Usage: %s image.fff xsizei ysizei fwhmr betar\n",argv[0]); 
   printf("       ampr coord(x y int)\n");
   exit(0);
} /* if */


strcpy(name,argv[1]);
xsize=atoi(argv[2]);
ysize=atoi(argv[3]);
fwhm=atof(argv[4]);
beta=atof(argv[5]);
amp0=atof(argv[6]);
strcpy(filename,argv[7]);

if (beta < 1.8) {
   printf("Warning: beta < 1.8, program terminated !\n");
   exit(0);
} /* if */


if ((infile = fopen(filename,"r")) == NULL) {
   printf("File not found.\n");
   exit(0);
} /* if */

if ((imfile = fopen(name,"wb")) == NULL) {
   printf("File not found.\n");
   exit(0);
} /* if */


eps=0.0005;

gamma=4.0*(exp(log(2.0)/beta)-1.0);
alfa=sqrt((exp(log(eps)/(1.0-beta))-1.0)/gamma);

rr=fwhm*alfa;

rri=(int)(rr+1.5);

kh=2*rri+1;
kw=kh;

printf("%8d\n",kh);

kw1=kw+1;
kh1=kh+1;

psf1=(double **)calloc(kw1+1,sizeof(double*));
mpsf1=(double *)calloc((kw1+1)*(kh1+1),sizeof(double));

psf2=(double **)calloc(kw1+1,sizeof(double*));
mpsf2=(double *)calloc((kw1+1)*(kh1+1),sizeof(double));

for (i=0; i<=kw1; i++) {
   psf1[i]=mpsf1;
   psf2[i]=mpsf2;
   mpsf1=mpsf1+kh1+1;
   mpsf2=mpsf2+kh1+1;
} /* for i */
 

fimage=(float **)calloc(ysize,sizeof(float*));
mpfimage=(float *)calloc(xsize*ysize,sizeof(float));

for (i=0; i<ysize; i++) {
   fimage[i]=mpfimage;
   mpfimage=mpfimage+xsize;
} /* for i */


for (j=0; j<ysize; j++) {
   for (i=0; i<xsize; i++) {
      fimage[j][i]=0.0;
   } /* for j */
} /* for i */


kw2=kw/2+1;
kh2=kh/2+1;


time1=clock();


while (fscanf(infile,"%lf %lf %lf",&x,&y,&inte) != EOF) {

      xi=(int)(x);
      yi=(int)(y);
      dx=x-(double)(xi);
      dy=y-(double)(yi);

      dx=1.0-dx;
      dy=1.0-dy;

      setmoff(psf1,psf2,kw,kh,dx,dy,fwhm,beta);

      sum=amp0*inte;

      for (i=1; i<=kh; i++) {
         for (j=1; j<=kw; j++) {
             i2=i+xi-kw2+1;
             j2=j+yi-kh2+1;
             if ((i2 >= 0) && (i2 < xsize)) {
                if ((j2 >= 0) && (j2 < ysize)) {
                   fftemp=fimage[j2][i2]+(float)(sum*psf2[i][j]);
                   fimage[j2][i2]=fftemp;
                } /* if */
             } /* if */
         } /* for j */
      } /* for i */
      

} /* while */

xys[1]=xsize;
xys[2]=ysize;


fwrite(&xys[1],sizeof(int),2,imfile);
 

for(j=0; j<ysize; j++) {
   for (i=0; i<xsize; i++) {
      dd[1]=fimage[j][i];
      fwrite(&dd[1],sizeof(float),1,imfile);
   } /* for j */
} /* for i */

timediff=clock()-time1;

printf("%10.2f\n",(double)(timediff)/timetic);

fclose(infile);
fclose(imfile);

} /* End  */
