/***************************************************************************/
/*                                                                         */
/*                                                                         */
/* SSPHOT A Photometry Package                                             */
/* ===========================                                             */
/*                                                                         */
/* This C-program is part of the SSPHOT photometry package.                */
/* See the links below for more information and documentation.             */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package and the documentation from the web-page:    */
/*                                                                         */
/* http://www.spaennare.se/ssphot.html                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

/*************************************************************/
/*  poissonf.c  SS Dec 97                                    */
/*                                                           */
/*************************************************************/

#include <math.h>
#include <stdio.h> 
#include <stdlib.h>
#include <unistd.h>
#include  <time.h>

double timetic=(double)(CLOCKS_PER_SEC);

double pi=3.14159265358979323;


#define IM1 2147483563
#define IM2 2147483399
#define AM (1.0/IM1)
#define IMM1 (IM1-1)
#define IA1 40014
#define IA2 40692
#define IQ1 53668
#define IQ2 52774
#define IR1 12211
#define IR2 3791
#define NTAB 32
#define NDIV (1+IMM1/NTAB)
#define EPS 1.2e-7
#define RNMX (1.0-EPS)

double ran2(idum)
long *idum;
{
   int j;
   long k;
   static long idum2=123456789;
   static long iy=0; 
   static long iv[NTAB];
   double temp;

   if (*idum <= 0) {
      if (-(*idum) < 1) {
         *idum=1;
      }
      else {
         *idum=-(*idum);
      } /* if */
      idum2=(*idum);
      for (j=NTAB+7; j>=0; j--) {
         k=(*idum)/IQ1;
	 *idum=IA1*(*idum-k*IQ1)-k*IR1;
	 if (*idum < 0) {
	    *idum += IM1;
	 } /* if */
	 if (j < NTAB) {
	    iv[j]=*idum;
	 } /* if */
      } /* for j */
      iy=iv[0];
   } /* if */
   k=(*idum)/IQ1;
   *idum=IA1*(*idum-k*IQ1)-k*IR1;
   if (*idum < 0) {
      *idum += IM1;
   } /* if */
   k=idum2/IQ2;
   idum2=IA2*(idum2-k*IQ2)-k*IR2;
   if (idum2 < 0) {
      idum2 += IM2;
   } /* if */
   j=iy/NDIV;
   iy=iv[j]-idum2;
   iv[j] = *idum;
   if (iy < 1) {
      iy += IMM1;
   } /* if */
   temp=AM*(double)(iy);
   if (temp > RNMX) {
      return(RNMX);
   }
   else {
      return(temp);
   } /* if */

} /* ran2 */


double gammln(xx)
double xx;
{
   int j;

   double x,y,tmp,ser,temp;

   double cof[6];

   cof[0]=76.18009172947146;
   cof[1]=-86.50532032941677;
   cof[2]=24.01409824083091;
   cof[3]=-1.231739572450155;
   cof[4]=0.1208650973866179e-2;
   cof[5]=-0.5395239384953e-5;

   x=xx;
   y=xx;
   tmp=x+5.5;
   tmp -= (x+0.5)*log(tmp);
   ser=1.000000000190015;

   for (j=0; j<=5; j++) {
      y=y+1.0;
      ser += cof[j]/y;
   } /* for j */

   temp=-tmp+log(2.5066282746310005*ser/x);

   return(temp);

} /* end gammln */


double poidev(xm,idum)
double xm;
long *idum;
{
   static double sq,alxm,g,oldm=(-1.0);

   double em,t,y;


   if (xm < 12.0) {
      if (xm != oldm) {
          oldm=xm;
          g=exp(-xm);
      } /* if */
      em=-1.0;
      t=1.0;
      do {
         ++em;
         t *= ran2(idum);
      } while (t > g);
   } else {
       if (xm != oldm) {
          oldm=xm;
          sq=sqrt(2.0*xm);
          alxm=log(xm);
          g=xm*alxm-gammln(xm+1.0);
       }
       do {
          do {
             y=tan(pi*ran2(idum));
             em=sq*y+xm;
          } while (em < 0.0);
          em=floor(em);
          t=0.9*(1.0+y*y)*exp(em*alxm-gammln(em+1.0)-g);
       } while (ran2(idum) >t);
   } /* else */

   return(em);

} /* end poidev */


int main(argc,argv) 
int argc;
char *argv[];
{

   FILE *infile,*outfile;

   int i,j,xsize,ysize;

   long seed;
   long idum;

   double gain,gaininv,temp,temp2;

   int xys[3];

   float fff[3];

   clock_t time1,timediff1;

   if (argc<4) {
      printf("Usage: %s inimage.fff outimage.fff seedi gainr\n",argv[0]);
      exit(0);
   } /* if */
   
   if ((infile = fopen(argv[1],"rb")) == NULL) {
      printf("File not found.\n");
      exit(0);
   } /* if */

   outfile=fopen(argv[2],"wb");

   seed=atoi(argv[3]);
   gain=atof(argv[4]);

   gaininv=1.0/gain;

   idum=-abs(seed)-1;

   fread(&xys[1],sizeof(int),2,infile);

   fwrite(&xys[1],sizeof(int),2,outfile);

   xsize=xys[1];
   ysize=xys[2];

   time1=clock();

   for (i=0; i<ysize; i++) {
      for (j=0; j<xsize; j++) {

         fread(&fff[1],sizeof(float),1,infile);

	 temp=gain*(double)(fff[1]);

	 temp2=poidev(temp,&idum);

	 fff[1]=(float)(gaininv*temp2);

	 fwrite(&fff[1],sizeof(float),1,outfile);

      } /* for j */
   } /* for i */  

   timediff1=clock()-time1;

   printf("%8.2f\n",(double)(timediff1)/timetic);

   fclose(outfile);
   fclose(infile);

} /* End */
