/***************************************************************************/
/*                                                                         */
/*                                                                         */
/* SSPHOT A Photometry Package                                             */
/* ===========================                                             */
/*                                                                         */
/* This C-program is part of the SSPHOT photometry package.                */
/* See the links below for more information and documentation.             */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package and the documentation from the web-page:    */
/*                                                                         */
/* http://www.spaennare.se/ssphot.html                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

/*************************************************************/
/*                                                           */
/* chist.c  SS 980725                                        */
/*                                                           */
/*************************************************************/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>

int main(argc,argv) 
int argc;
char *argv[];

{
   
   FILE *infile;

   int ll,i,imax,imin,sum;

   double x,y,inte,lt;

   int hh[31];

   if (argc < 2) {
      printf("Usage: %s file(x y int)\n",argv[0]);
      exit(0);
   } /* if */
   

   if ((infile = fopen(argv[1],"r")) == NULL) {
      printf("File not found.\n");
      exit(0);
   } /* if */

   lt=1.0/log(10.0);

   for (i=1; i<=30; i++) {
      hh[i]=0;
   } /* for i */

   while (fscanf(infile,"%lf %lf %lf",&x,&y,&inte) != EOF) {
      ll=(int)(lt*log(inte))+15;
      hh[ll]++;
   } /* while */

   i=30;

   while (hh[i] == 0) {
      i--;
   } /* while */

   imax=i;

   i=1;

   while (hh[i] == 0) {
      i++;
   } /* while */

   imin=i;

   printf(" log(int)          n      n_cum\n");

   sum=0;

   for (i=imax; i>=imin; i--) {
       sum=sum+hh[i];
       ll=i-15;
       printf("%4d %4d %10d %10d\n",ll,ll+1,hh[i],sum);
   } /* for i */

 
} /* End */
