/***************************************************************************/
/*                                                                         */
/*                                                                         */
/* SSPHOT A Photometry Package                                             */
/* ===========================                                             */
/*                                                                         */
/* This C-program is part of the SSPHOT photometry package.                */
/* See the links below for more information and documentation.             */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package and the documentation from the web-page:    */
/*                                                                         */
/* http://www.spaennare.se/ssphot.html                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

/****************************************************************/
/* FFT converted from FORTRAN to C with f2c by                  */
/* Stefan Spaennare Nov 96                                      */
/* Double precision.                                            */
/****************************************************************/

#define max(a,b) ((a) >= (b) ? (a) : (b))

int dfourt_(data, nn, ndim, isign, iform, work)
double *data;
int *nn, *ndim, *isign, *iform;
double *work;
{
    /* System generated locals */
    int i__1, i__2, i__3, i__4, i__5, i__6, i__7, i__8, i__9, i__10;

    /* Builtin functions */
    double sqrt(), cos(), sin();

    /* Local variables */
    double difi;
    int kdif, idim;
    double difr;
    int idiv, imin, irem, imax, jmax, ipar, lmax, mmax, kmin, jmin;
    double sumi;
    int ntot;
    double sumr;
    int ntwo, np2hf, i1max, i1rng, i2max, j2max, j3max, inon2, 
	    non2p, i, j, np1tw, l, m, n, icase, ifact[32], nhalf, ifmin;
    double theta, oldsi, tempi, rthlf, thetm, wmini, oldsr;
    int kstep;
    double tempr;
    int nprev, i1;
    double wtemp, wminr, stmpi;
    int i2, i3, iquot;
    double twopi;
    int j1, j2;
    double stmpr, wstpi;
    int j3, k1, k2, k3, nwork, k4;
    double wstpr, twowr;
    int if__;
    double si, wi, sr, wr;
    int np0;
    double t2i, t3i, t4i, w2i, w3i, u1i, u2i, u3i, u4i, t2r, t3r, 
	    t4r, w2r, w3r, u1r, u2r, u3r, u4r;
    int np1, np2, ifp1, ifp2;


/* Double Precision version of FOURT.F */

/* Richard Hook, ST-ECF, October 1992 */

/* -------------------Start of original code (modifed to be more readable)
 */

/*  June 30, 1983 */

/*  Watch out for INTEGER*4 on VAX!! NN, ISIGN and IFORM must all */
/*  be dimensioned INTEGER*4 */

/*     The Fast Fourier Transform in USASI basic FORTRAN */
/*     ------------------------------------------------- */

/*     TRANSFORM(J1,J2,...) = SUM(DATA(I1,I2,...)*W1**((I1-1)*(J1-1)) */
/*                                 *W2**((I2-1)*(J2-1))*...), */

/*     where I1 and J1 run from 1 to NN(1) and W1=EXP(ISIGN*2*PI* */
/*     SQRT(-1)/NN(1)), etc.  there is no limit on the dimensionality */
/*     (number of subscripts) of the data array.  If an inverse */
/*     transform (ISIGN=+1) is performed upon an array of transformed */
/*     (ISIGN=-1) data, the original data will reappear, */
/*     multiplied by NN(1)*NN(2)*...  The array of input data may be */
/*     real or complex, at the programmers option, with a saving of */
/*     up to forty per cent in running time for real over complex. */
/*     (for fastest transform of real data, NN(1) should be even.) */
/*     the transform values are always complex, and are returned in the */
/*     original array of data, replacing the input data.  the length */
/*     of each dimension of the data array may be any integer.  the */
/*     program runs faster on composite integers than on primes, and is */
/*     particularly fast on numbers rich in factors of two. */

/*     Timing is in fact given by the following formula.  Let NTOT be the 
*/
/*     total number of points (real or complex) in the data array, that */
/*     is, NTOT=NN(1)*NN(2)*...  decompose NTOT into its prime factors, */
/*     such as 2**K2 * 3**K3 * 5**K5 * ...  Let SUM2 be the sum of all */
/*     the factors of two in NTOT, that is, SUM2 = 2*K2.  Let SUMF be */
/*     the sum of all other factors of NTOT, that is, SUMF = 3*K3+5*K5+.. 
*/
/*     The time taken by a multidimensional transform on these NTOT data 
*/
/*     point add time = six microseconds), T = 3000 + NTOT*(600+40*SUM2+ 
*/
/*     is T = T0 + NTOT*(T1+T2*SUM2+T3*SUMF). On the CDC 3300 (floating */
/*     175*SUMF) microseconds on complex data. */

/*     Implementation of the definition by summation will run in a time */
/*     proportional to NTOT*(NN(1)+NN(2)+...). For highly composite NTOT 
*/
/*     the savings offered by this program can be dramatic. A one-dimen- 
*/
/*     sional array 4000 in length will be transformed in 4000*(600+ */
/*     40*(2+2+2+2+2)+175*(5+5+5)) = 14.5 seconds versus about 4000* */
/*     4000*175 = 2800 seconds for the straightforward technique. */

/*     [Note added by Richard Hook, October 1992:  as a rough guide  ] */
/*     [the time taken for a real and imaginary FFT of a 512*512     ] */
/*     [array is about 8s on a SPARCStation 2                        ] */

/*     The Fast Fourier algorithm places two restrictions upon the */
/*     nature of the data beyond the usual restriction that */
/*     the data form one cycle of a periodic function.  They are-- */
/*     1.  The number of input data and the number of transform values */
/*     must be the same. */
/*     2. Considering the data to be in the time domain, */
/*     they must be equi-spaced at intervals of DT.  Further, the trans- 
*/
/*     form values, considered to be in frequency space, will be equi- */
/*     spaced from 0 to 2*PI*(NN(I)-1)/(NN(I)*DT) at intervals of */
/*     2*PI/(NN(I)*DT) for each dimension of length NN(I).  Of course, */
/*     dt need not be the same for every dimension. */

/*     The calling sequence is-- */

/*     CALL DFOURT(DATA,NN,NDIM,ISIGN,IFORM,WORK) */

/*     DATA is the array used to hold the real and imaginary parts */
/*     of the data on input and the transform values on output.  It */
/*     is a multidimensional floating point array, with the real and */
/*     imaginary parts of a datum stored immediately adjacent in storage 
*/
/*     (such as FORTRAN IV places them).  The extent of each dimension */
/*     is given in the integer array NN, of length NDIM.  ISIGN is -1 */
/*     to indicate a forward transform (exponential sign is -) and +1 */
/*     for an inverse transform (sign is +).  IFORM is +1 if the data and 
*/
/*     the transform values are complex.  It is 0 if the data are real */
/*     but the transform values are complex.  If it is 0, the imaginary */
/*     parts of the data should be set to zero.  As explained above, the 
*/
/*     transform values are always complex and are stored in array data. 
*/
/*     work is an array used for working storage.  It is not necessary */
/*     if all the dimensions of the data are powers of two.  In this case 
*/
/*     it may be replaced by 0 in the calling sequence.  Thus, use of */
/*     powers of two can free a good deal of storage.  If any dimension */
/*     is not a power of two, this array must be supplied.  It is */
/*     floating point, one dimensional of length equal to twice the */
/*     largest array dimension (i.e., NN(I) ) that is not a power of */
/*     two.  Therefore, in one dimension for a non power of two, */
/*     work occupies as many storage locations as data.  If supplied, */
/*     work must not be the same array as data.  All subscripts of all */
/*     arrays begin at one. */

/*     Example 1.  Three-dimensional forward Fourier Transform of a */
/*     complex array dimensioned 32 by 25 by 13 in FORTRAN IV. */

/*     DOUBLE PRECISION DATA(32,25,13),WORK(50),NN(3) */
/*     COMPLEX DATA */
/*     DATA NN/32,25,13/ */
/*     DO 1 I=1,32 */
/*     DO 1 J=1,25 */
/*     DO 1 K=1,13 */
/*  1  DATA(I,J,K)=COMPLEX VALUE */
/*     CALL DFOURT(DATA,NN,3,-1,1,WORK) */

/*     Example 2.  One-dimensional forward transform of a real array of */
/*     length 64 in FORTRAN II. */

/*     DOUBLE PRECISION DATA(2,64) */
/*     DO 2 I=1,64 */
/*     DATA(1,I)=REAL PART */
/*  2  DATA(2,I)=0. */
/*     CALL DFOURT(DATA,64,1,-1,0,0) */

/*     There are no error messages or error halts in this program.  The */
/*     program returns immediately if ndim or any nn(i) is less than one. 
*/

/*     Program by Norman Brenner from the BASIC program by Charles */
/*     Rader (both of MIT Lincoln Laboratory).  May 1967.  The idea */
/*     for the digit reversal was suggested by Ralph Alter (also MIT LL). 
*/
/*     This is the fastest and most versatile version of the FFT known */
/*     to the author.  A program called four2 is available that also */
/*     performs the fast fourier transform and is written in usasi basic 
*/
/*     fortran.  It is about one third as long and restricts the */
/*     dimensions of the input array (which must be complex) to be powers 
*/
/*     of two.  Another program, called four1, is one tenth as long and */
/*     runs two thirds as fast on a one-dimensional complex array whose */
/*     length is a power of two. */

/*     Reference-- */
/*     Fast Fourier Transforms for Fun and Profit, W. Gentleman and */
/*     G. Sande, 1966 Fall Joint Computer Conference. */

/*     The work reported in this document was performed at Lincoln Lab- */
/*     oratory, a center for research operated by Massachusetts Institute 
*/
/*     of Technology, with the support of the U.S. Air Force under */
/*     contract af 19(628)-5167. */

/* -------------End of original comment block. */


/* Initialise constants */
    /* Parameter adjustments */
    --work;
    --nn;
    --data;

    /* Function Body */
    twopi = 6.28318530717959;
    rthlf = sqrt(2.) / 2.;
/* Start of executable code */
    if (*ndim - 1 >= 0) {
	goto L1;
    } else {
	goto L920;
    }
L1:
    ntot = 2;
    i__1 = *ndim;
    for (idim = 1; idim <= i__1; ++idim) {
	if (nn[idim] <= 0) {
	    goto L920;
	} else {
	    goto L2;
	}
L2:
	ntot *= nn[idim];
    }

/*     main loop for each dimension */

    np1 = 2;
    i__1 = *ndim;
    for (idim = 1; idim <= i__1; ++idim) {
	n = nn[idim];
	np2 = np1 * n;
	if ((i__2 = n - 1) < 0) {
	    goto L920;
	} else if (i__2 == 0) {
	    goto L900;
	} else {
	    goto L5;
	}

/*     is n a power of two and if not, what are its factors */

L5:
	m = n;
	ntwo = np1;
	if__ = 1;
	idiv = 2;
L10:
	iquot = m / idiv;
	irem = m - idiv * iquot;
	if (iquot - idiv >= 0) {
	    goto L11;
	} else {
	    goto L50;
	}
L11:
	if (irem != 0) {
	    goto L20;
	} else {
	    goto L12;
	}
L12:
	ntwo += ntwo;
	ifact[if__ - 1] = idiv;
	++if__;
	m = iquot;
	goto L10;
L20:
	idiv = 3;
	inon2 = if__;
L30:
	iquot = m / idiv;
	irem = m - idiv * iquot;
	if (iquot - idiv >= 0) {
	    goto L31;
	} else {
	    goto L60;
	}
L31:
	if (irem != 0) {
	    goto L40;
	} else {
	    goto L32;
	}
L32:
	ifact[if__ - 1] = idiv;
	++if__;
	m = iquot;
	goto L30;
L40:
	idiv += 2;
	goto L30;
L50:
	inon2 = if__;
	if (irem != 0) {
	    goto L60;
	} else {
	    goto L51;
	}
L51:
	ntwo += ntwo;
	goto L70;
L60:
	ifact[if__ - 1] = m;
L70:
	non2p = np2 / ntwo;

/*     Separate four cases-- */
/*        1. Complex transform */
/*        2. Real transform for the 2nd, 3rd, etc. dimension.  method
-- */
/*           transform half the data, supplying the other half by con-
 */
/*           jugate symmetry. */
/*        3. Real transform for the 1st dimension, n odd.  method-- */
/*           set the imaginary parts to zero. */
/*        4. Real transform for the 1st dimension, n even.  method-- 
*/
/*           transform a complex array of length n/2 whose real parts 
*/
/*           are the even numbered real values and whose imaginary par
ts */
/*           are the odd numbered real values.  separate and supply */
/*           the second half by conjugate symmetry. */

	icase = 1;
	ifmin = 1;
	i1rng = np1;
	if (idim - 4 >= 0) {
	    goto L100;
	} else {
	    goto L74;
	}
L74:
	if (*iform <= 0) {
	    goto L71;
	} else {
	    goto L100;
	}
L71:
	icase = 2;
	i1rng = np0 * (nprev / 2 + 1);
	if (idim - 1 <= 0) {
	    goto L72;
	} else {
	    goto L100;
	}
L72:
	icase = 3;
	i1rng = np1;
	if (ntwo - np1 <= 0) {
	    goto L100;
	} else {
	    goto L73;
	}
L73:
	icase = 4;
	ifmin = 2;
	ntwo /= 2;
	n /= 2;
	np2 /= 2;
	ntot /= 2;
	i = 1;
	i__2 = ntot;
	for (j = 1; j <= i__2; ++j) {
	    data[j] = data[i];
/* L80: */
	    i += 2;
	}

/*     Shuffle data by bit reversal, since n=2**k.  as the shuffling 
*/
/*     can be done by simple interchange, no working array is needed 
*/

L100:
	if (non2p - 1 <= 0) {
	    goto L101;
	} else {
	    goto L200;
	}
L101:
	np2hf = np2 / 2;
	j = 1;
	i__2 = np2;
	i__3 = np1;
	for (i2 = 1; i__3 < 0 ? i2 >= i__2 : i2 <= i__2; i2 += i__3) {
	    if (j - i2 >= 0) {
		goto L130;
	    } else {
		goto L121;
	    }
L121:
	    i1max = i2 + np1 - 2;
	    i__4 = i1max;
	    for (i1 = i2; i1 <= i__4; i1 += 2) {
		i__5 = ntot;
		i__6 = np2;
		for (i3 = i1; i__6 < 0 ? i3 >= i__5 : i3 <= i__5; i3 += i__6) 
			{
		    j3 = j + i3 - i2;
		    tempr = data[i3];
		    tempi = data[i3 + 1];
		    data[i3] = data[j3];
		    data[i3 + 1] = data[j3 + 1];
		    data[j3] = tempr;
/* L125: */
		    data[j3 + 1] = tempi;
		}
	    }
L130:
	    m = np2hf;
L140:
	    if (j - m <= 0) {
		goto L150;
	    } else {
		goto L141;
	    }
L141:
	    j -= m;
	    m /= 2;
	    if (m - np1 >= 0) {
		goto L140;
	    } else {
		goto L150;
	    }
L150:
	    j += m;
	}
	goto L300;

/*     Shuffle data by digit reversal for general n */

L200:
	nwork = n << 1;
	i__3 = np1;
	for (i1 = 1; i1 <= i__3; i1 += 2) {
	    i__2 = ntot;
	    i__6 = np2;
	    for (i3 = i1; i__6 < 0 ? i3 >= i__2 : i3 <= i__2; i3 += i__6) {
		j = i3;
		i__5 = nwork;
		for (i = 1; i <= i__5; i += 2) {
		    if (icase - 3 != 0) {
			goto L210;
		    } else {
			goto L220;
		    }
L210:
		    work[i] = data[j];
		    work[i + 1] = data[j + 1];
		    goto L240;
L220:
		    work[i] = data[j];
		    work[i + 1] = 0.;
L240:
		    ifp2 = np2;
		    if__ = ifmin;
L250:
		    ifp1 = ifp2 / ifact[if__ - 1];
		    j += ifp1;
		    if (j - i3 - ifp2 >= 0) {
			goto L255;
		    } else {
			goto L260;
		    }
L255:
		    j -= ifp2;
		    ifp2 = ifp1;
		    ++if__;
		    if (ifp2 - np1 <= 0) {
			goto L260;
		    } else {
			goto L250;
		    }
L260:
		    ;
		}
		i2max = i3 + np2 - np1;
		i = 1;
		i__5 = i2max;
		i__4 = np1;
		for (i2 = i3; i__4 < 0 ? i2 >= i__5 : i2 <= i__5; i2 += i__4) 
			{
		    data[i2] = work[i];
		    data[i2 + 1] = work[i + 1];
/* L270: */
		    i += 2;
		}
	    }
	}

/*     Main loop for factors of two. */
/*     W=EXP(ISIGN*2*PI*SQRT(-1)*M/(4*MMAX)).  Check for W=ISIGN*SQRT(
-1) */
/*     and repeat for W=W*(1+ISIGN*SQRT(-1))/SQRT(2). */

L300:
	if (ntwo - np1 <= 0) {
	    goto L600;
	} else {
	    goto L305;
	}
L305:
	np1tw = np1 + np1;
	ipar = ntwo / np1;
L310:
	if ((i__4 = ipar - 2) < 0) {
	    goto L350;
	} else if (i__4 == 0) {
	    goto L330;
	} else {
	    goto L320;
	}
L320:
	ipar /= 4;
	goto L310;
L330:
	i__4 = i1rng;
	for (i1 = 1; i1 <= i__4; i1 += 2) {
	    i__5 = ntot;
	    i__6 = np1tw;
	    for (k1 = i1; i__6 < 0 ? k1 >= i__5 : k1 <= i__5; k1 += i__6) {
		k2 = k1 + np1;
		tempr = data[k2];
		tempi = data[k2 + 1];
		data[k2] = data[k1] - tempr;
		data[k2 + 1] = data[k1 + 1] - tempi;
		data[k1] += tempr;
/* L340: */
		data[k1 + 1] += tempi;
	    }
	}
L350:
	mmax = np1;
L360:
	if (mmax - ntwo / 2 >= 0) {
	    goto L600;
	} else {
	    goto L370;
	}
L370:
/* Computing MAX */
	i__6 = np1tw, i__5 = mmax / 2;
	lmax = max(i__6,i__5);
	i__6 = lmax;
	i__5 = np1tw;
	for (l = np1; i__5 < 0 ? l >= i__6 : l <= i__6; l += i__5) {
	    m = l;
	    if (mmax - np1 <= 0) {
		goto L420;
	    } else {
		goto L380;
	    }
L380:
	    theta = -twopi * (double) l / (double) (mmax << 2);
	    if (*isign >= 0) {
		goto L390;
	    } else {
		goto L400;
	    }
L390:
	    theta = -theta;
L400:
	    wr = cos(theta);
	    wi = sin(theta);
L410:
	    w2r = wr * wr - wi * wi;
	    w2i = wr * 2. * wi;
	    w3r = w2r * wr - w2i * wi;
	    w3i = w2r * wi + w2i * wr;
L420:
	    i__4 = i1rng;
	    for (i1 = 1; i1 <= i__4; i1 += 2) {
		kmin = i1 + ipar * m;
		if (mmax - np1 <= 0) {
		    goto L430;
		} else {
		    goto L440;
		}
L430:
		kmin = i1;
L440:
		kdif = ipar * mmax;
L450:
		kstep = kdif << 2;
		if (kstep - ntwo <= 0) {
		    goto L460;
		} else {
		    goto L530;
		}
L460:
		i__2 = ntot;
		i__3 = kstep;
		for (k1 = kmin; i__3 < 0 ? k1 >= i__2 : k1 <= i__2; k1 += 
			i__3) {
		    k2 = k1 + kdif;
		    k3 = k2 + kdif;
		    k4 = k3 + kdif;
		    if (mmax - np1 <= 0) {
			goto L470;
		    } else {
			goto L480;
		    }
L470:
		    u1r = data[k1] + data[k2];
		    u1i = data[k1 + 1] + data[k2 + 1];
		    u2r = data[k3] + data[k4];
		    u2i = data[k3 + 1] + data[k4 + 1];
		    u3r = data[k1] - data[k2];
		    u3i = data[k1 + 1] - data[k2 + 1];
		    if (*isign >= 0) {
			goto L472;
		    } else {
			goto L471;
		    }
L471:
		    u4r = data[k3 + 1] - data[k4 + 1];
		    u4i = data[k4] - data[k3];
		    goto L510;
L472:
		    u4r = data[k4 + 1] - data[k3 + 1];
		    u4i = data[k3] - data[k4];
		    goto L510;
L480:
		    t2r = w2r * data[k2] - w2i * data[k2 + 1];
		    t2i = w2r * data[k2 + 1] + w2i * data[k2];
		    t3r = wr * data[k3] - wi * data[k3 + 1];
		    t3i = wr * data[k3 + 1] + wi * data[k3];
		    t4r = w3r * data[k4] - w3i * data[k4 + 1];
		    t4i = w3r * data[k4 + 1] + w3i * data[k4];
		    u1r = data[k1] + t2r;
		    u1i = data[k1 + 1] + t2i;
		    u2r = t3r + t4r;
		    u2i = t3i + t4i;
		    u3r = data[k1] - t2r;
		    u3i = data[k1 + 1] - t2i;
		    if (*isign >= 0) {
			goto L500;
		    } else {
			goto L490;
		    }
L490:
		    u4r = t3i - t4i;
		    u4i = t4r - t3r;
		    goto L510;
L500:
		    u4r = t4i - t3i;
		    u4i = t3r - t4r;
L510:
		    data[k1] = u1r + u2r;
		    data[k1 + 1] = u1i + u2i;
		    data[k2] = u3r + u4r;
		    data[k2 + 1] = u3i + u4i;
		    data[k3] = u1r - u2r;
		    data[k3 + 1] = u1i - u2i;
		    data[k4] = u3r - u4r;
/* L520: */
		    data[k4 + 1] = u3i - u4i;
		}
		kdif = kstep;
		kmin = (kmin - i1 << 2) + i1;
		goto L450;
L530:
		;
	    }
	    m += lmax;
	    if (m - mmax <= 0) {
		goto L540;
	    } else {
		goto L570;
	    }
L540:
	    if (*isign >= 0) {
		goto L560;
	    } else {
		goto L550;
	    }
L550:
	    tempr = wr;
	    wr = (wr + wi) * rthlf;
	    wi = (wi - tempr) * rthlf;
	    goto L410;
L560:
	    tempr = wr;
	    wr = (wr - wi) * rthlf;
	    wi = (tempr + wi) * rthlf;
	    goto L410;
L570:
	    ;
	}
	ipar = 3 - ipar;
	mmax += mmax;
	goto L360;

/*     Main loop for factors not equal to two. */
/*     W=EXP(ISIGN*2*PI*SQRT(-1)*(J1+J2-I3-1)/IFP2) */

L600:
	if (non2p - 1 <= 0) {
	    goto L700;
	} else {
	    goto L601;
	}
L601:
	ifp1 = ntwo;
	if__ = inon2;
L610:
	ifp2 = ifact[if__ - 1] * ifp1;
	theta = -twopi / (double) ifact[if__ - 1];
	if (*isign >= 0) {
	    goto L611;
	} else {
	    goto L612;
	}
L611:
	theta = -theta;
L612:
	wstpr = cos(theta);
	wstpi = sin(theta);
	i__5 = ifp1;
	i__6 = np1;
	for (j1 = 1; i__6 < 0 ? j1 >= i__5 : j1 <= i__5; j1 += i__6) {
	    thetm = -twopi * (double) (j1 - 1) / (double) ifp2;
	    if (*isign >= 0) {
		goto L613;
	    } else {
		goto L614;
	    }
L613:
	    thetm = -thetm;
L614:
	    wminr = cos(thetm);
	    wmini = sin(thetm);
	    i1max = j1 + i1rng - 2;
	    i__4 = i1max;
	    for (i1 = j1; i1 <= i__4; i1 += 2) {
		i__3 = ntot;
		i__2 = np2;
		for (i3 = i1; i__2 < 0 ? i3 >= i__3 : i3 <= i__3; i3 += i__2) 
			{
		    i = 1;
		    wr = wminr;
		    wi = wmini;
		    j2max = i3 + ifp2 - ifp1;
		    i__7 = j2max;
		    i__8 = ifp1;
		    for (j2 = i3; i__8 < 0 ? j2 >= i__7 : j2 <= i__7; j2 += 
			    i__8) {
			twowr = wr + wr;
			j3max = j2 + np2 - ifp2;
			i__9 = j3max;
			i__10 = ifp2;
			for (j3 = j2; i__10 < 0 ? j3 >= i__9 : j3 <= i__9; j3 
				+= i__10) {
			    jmin = j3 - j2 + i3;
			    j = jmin + ifp2 - ifp1;
			    sr = data[j];
			    si = data[j + 1];
			    oldsr = 0.;
			    oldsi = 0.;
			    j -= ifp1;
L620:
			    stmpr = sr;
			    stmpi = si;
			    sr = twowr * sr - oldsr + data[j];
			    si = twowr * si - oldsi + data[j + 1];
			    oldsr = stmpr;
			    oldsi = stmpi;
			    j -= ifp1;
			    if (j - jmin <= 0) {
				goto L621;
			    } else {
				goto L620;
			    }
L621:
			    work[i] = wr * sr - wi * si - oldsr + data[j];
			    work[i + 1] = wi * sr + wr * si - oldsi + data[j 
				    + 1];
/* L630: */
			    i += 2;
			}
			wtemp = wr * wstpi;
			wr = wr * wstpr - wi * wstpi;
/* L640: */
			wi = wi * wstpr + wtemp;
		    }
		    i = 1;
		    i__8 = j2max;
		    i__7 = ifp1;
		    for (j2 = i3; i__7 < 0 ? j2 >= i__8 : j2 <= i__8; j2 += 
			    i__7) {
			j3max = j2 + np2 - ifp2;
			i__10 = j3max;
			i__9 = ifp2;
			for (j3 = j2; i__9 < 0 ? j3 >= i__10 : j3 <= i__10; 
				j3 += i__9) {
			    data[j3] = work[i];
			    data[j3 + 1] = work[i + 1];
/* L650: */
			    i += 2;
			}
		    }
		}
	    }
	}
	++if__;
	ifp1 = ifp2;
	if (ifp1 - np2 >= 0) {
	    goto L700;
	} else {
	    goto L610;
	}

/*     Complete a real transform in the 1st dimension, N even, by con-
 */
/*     jugate symmetries. */

L700:
	switch ((int)icase) {
	    case 1:  goto L900;
	    case 2:  goto L800;
	    case 3:  goto L900;
	    case 4:  goto L701;
	}
L701:
	nhalf = n;
	n += n;
	theta = -twopi / (double) n;
	if (*isign >= 0) {
	    goto L702;
	} else {
	    goto L703;
	}
L702:
	theta = -theta;
L703:
	wstpr = cos(theta);
	wstpi = sin(theta);
	wr = wstpr;
	wi = wstpi;
	imin = 3;
	jmin = (nhalf << 1) - 1;
	goto L725;
L710:
	j = jmin;
	i__9 = ntot;
	i__10 = np2;
	for (i = imin; i__10 < 0 ? i >= i__9 : i <= i__9; i += i__10) {
	    sumr = (data[i] + data[j]) / 2.;
	    sumi = (data[i + 1] + data[j + 1]) / 2.;
	    difr = (data[i] - data[j]) / 2.;
	    difi = (data[i + 1] - data[j + 1]) / 2.;
	    tempr = wr * sumi + wi * difr;
	    tempi = wi * sumi - wr * difr;
	    data[i] = sumr + tempr;
	    data[i + 1] = difi + tempi;
	    data[j] = sumr - tempr;
	    data[j + 1] = -difi + tempi;
/* L720: */
	    j += np2;
	}
	imin += 2;
	jmin += -2;
	wtemp = wr * wstpi;
	wr = wr * wstpr - wi * wstpi;
	wi = wi * wstpr + wtemp;
L725:
	if ((i__10 = imin - jmin) < 0) {
	    goto L710;
	} else if (i__10 == 0) {
	    goto L730;
	} else {
	    goto L740;
	}
L730:
	if (*isign >= 0) {
	    goto L740;
	} else {
	    goto L731;
	}
L731:
	i__10 = ntot;
	i__9 = np2;
	for (i = imin; i__9 < 0 ? i >= i__10 : i <= i__10; i += i__9) {
/* L735: */
	    data[i + 1] = -data[i + 1];
	}
L740:
	np2 += np2;
	ntot += ntot;
	j = ntot + 1;
	imax = ntot / 2 + 1;
L745:
	imin = imax - (nhalf << 1);
	i = imin;
	goto L755;
L750:
	data[j] = data[i];
	data[j + 1] = -data[i + 1];
L755:
	i += 2;
	j += -2;
	if (i - imax >= 0) {
	    goto L760;
	} else {
	    goto L750;
	}
L760:
	data[j] = data[imin] - data[imin + 1];
	data[j + 1] = (float)0.;
	if (i - j >= 0) {
	    goto L780;
	} else {
	    goto L770;
	}
L765:
	data[j] = data[i];
	data[j + 1] = data[i + 1];
L770:
	i += -2;
	j += -2;
	if (i - imin <= 0) {
	    goto L775;
	} else {
	    goto L765;
	}
L775:
	data[j] = data[imin] + data[imin + 1];
	data[j + 1] = (float)0.;
	imax = imin;
	goto L745;
L780:
	data[1] += data[2];
	data[2] = (float)0.;
	goto L900;

/*     Complete a real transform for the 2nd, 3rd, etc. dimension by 
*/
/*     conjugate symmetries. */

L800:
	if (i1rng - np1 >= 0) {
	    goto L900;
	} else {
	    goto L805;
	}
L805:
	i__9 = ntot;
	i__10 = np2;
	for (i3 = 1; i__10 < 0 ? i3 >= i__9 : i3 <= i__9; i3 += i__10) {
	    i2max = i3 + np2 - np1;
	    i__7 = i2max;
	    i__8 = np1;
	    for (i2 = i3; i__8 < 0 ? i2 >= i__7 : i2 <= i__7; i2 += i__8) {
		imax = i2 + np1 - 2;
		imin = i2 + i1rng;
		jmax = (i3 << 1) + np1 - imin;
		if (i2 - i3 <= 0) {
		    goto L820;
		} else {
		    goto L810;
		}
L810:
		jmax += np2;
L820:
		if (idim - 2 <= 0) {
		    goto L850;
		} else {
		    goto L830;
		}
L830:
		j = jmax + np0;
		i__2 = imax;
		for (i = imin; i <= i__2; i += 2) {
		    data[i] = data[j];
		    data[i + 1] = -data[j + 1];
/* L840: */
		    j += -2;
		}
L850:
		j = jmax;
		i__2 = imax;
		i__3 = np0;
		for (i = imin; i__3 < 0 ? i >= i__2 : i <= i__2; i += i__3) {
		    data[i] = data[j];
		    data[i + 1] = -data[j + 1];
/* L860: */
		    j -= np0;
		}
	    }
	}

/*     End of loop on each dimension */

L900:
	np0 = np1;
	np1 = np2;
/* L910: */
	nprev = n;
    }
L920:
    return 0;
} /* dfourt_ */

