/***************************************************************************/
/*                                                                         */
/*                                                                         */
/* SSPHOT A Photometry Package                                             */
/* ===========================                                             */
/*                                                                         */
/* This C-program is part of the SSPHOT photometry package.                */
/* See the links below for more information and documentation.             */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package and the documentation from the web-page:    */
/*                                                                         */
/* http://www.spaennare.se/ssphot.html                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

/*************************************************************/
/*                                                           */
/*   fsorts.c        By Stefan Spaennare                     */
/*   First version:  Dec 1995                                */
/*   Latest updates: Nov 1996, June 2002                     */
/*                                                           */
/*************************************************************/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>

double timetic=(double)(CLOCKS_PER_SEC);


int main(argc,argv)
int argc;
char *argv[];

{
   FILE *infile,*outfile;

   int i,j,n1,nn,i0,n,n2,tempi,mm,mm2;

   double r2,x1,y1,dx,dy,lt,mag;
   double rrej,rrej2;
   double rstep,rstep2;

   float temp,max,inte1,sig1;

   float data[6];

   float *x, *y, *inte, *bg, *sig;

   int *m;

   clock_t time1,timediff1;


   if (argc != 5) {
      printf("Usage: %s findsfile.bin outfile.asc rrejr rstepr\n",argv[0]);
      exit(0);
   } /* if */

   rrej=atof(argv[3]);
   rstep=atof(argv[4]);


   if ((infile = fopen(argv[1],"r")) == NULL) {
      printf("File not found.\n");
      exit(0);
   } /* if */

   rewind(infile);

   fseek(infile,0,SEEK_END);

   n=ftell(infile);

   n=n/(5*sizeof(float));

   printf("%9d\n",n);

   x=(float *)calloc(n+1,sizeof(float));
   y=(float *)calloc(n+1,sizeof(float));
   inte=(float *)calloc(n+1,sizeof(float));
   bg=(float *)calloc(n+1,sizeof(float));
   sig=(float *)calloc(n+1,sizeof(float));

   m=(int *)calloc(n+1,sizeof(int));

   rewind(infile);

   time1=clock();

   for (i=1; i<=n; i++) {
      fread(&data[1],sizeof(float),5,infile);
      x[i]=data[1];
      y[i]=data[2];
      inte[i]=data[3];
      bg[i]=data[4];
      sig[i]=data[5];
      m[i]=i;
   } /* for i */

   fclose(infile);

   outfile=fopen(argv[2],"w");

    lt=1.0/log(10.0);

    rrej2=rrej*rrej;
    rstep2=rstep*rstep;

    j=1;
    n1=1;

    while (n1 < n) {

       max=inte[m[n1]];   
       i0=n1;

       for (i=n1; i<=n; i++) {
          mm=m[i];
          temp=inte[mm];
          if (temp > max) {
             max=temp;
             i0=i;
          } /* if */
       } /* for i */

       if (i0 != n1) {
          tempi=m[i0]; m[i0]=m[n1]; m[n1]=tempi;
       } /* if */

       mm=m[n1];
       x1=(double)(x[mm]);
       y1=(double)(y[mm]);
       
       nn=n1+1;
       i0=nn;

       for (i=nn; i<=n; i++) {
            
           mm=m[i];
           dx=(double)(x[mm])-x1;
           dy=(double)(y[mm])-y1;

           r2=dx*dx+dy*dy;

           if (r2 < rrej2) {
              tempi=m[i0]; m[i0]=m[i]; m[i]=tempi;
              i0++;
           } /* if */ 
       } /* for i */

       mm=m[n1];
       sig1=sig[mm];
       n2=n1;

       for (i=nn; i<i0; i++) {

           mm=m[i];
           dx=(double)(x[mm])-x1;
           dy=(double)(y[mm])-y1;

           r2=dx*dx+dy*dy;

           if (r2 < rstep2) {
              mm=m[i];
              temp=sig[mm];
              if (temp < sig1) {
                 sig1=temp;
                 n2=i;
              } /* if */
           } /* if */ 

       } /* for i */

       mm=m[n1];
       mm2=m[n2];

       inte1=inte[mm];
       mag=-2.5*lt*log(inte1);
       fprintf(outfile,"%6d %8.3f %8.3f %13.3f %8.3f %9.3f %12.6f\n",
       j,x[mm2],y[mm2],inte1,mag,bg[mm],sig[mm]);


       n1=i0;
       j++;

   } /* while */ 


   fclose(outfile);

   timediff1=clock()-time1;

   printf("    nr    x-pos    y-pos           int      mag");
   printf("        bg        sigma\n");
   printf("\n");
   printf("    Printed to file: %s\n",argv[2]);

   printf("%9d\n",j-1);

   printf("%9.2f\n",(double)(timediff1)/timetic);
                
} /* End */
