/***************************************************************************/
/*                                                                         */
/*                                                                         */
/* SSPHOT A Photometry Package                                             */
/* ===========================                                             */
/*                                                                         */
/* This C-program is part of the SSPHOT photometry package.                */
/* See the links below for more information and documentation.             */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package and the documentation from the web-page:    */
/*                                                                         */
/* http://www.spaennare.se/ssphot.html                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

 
/************* 
>  imf2eps.c     to create imf2eps, which takes an image file, and con-
\*************   verts it into an encapsulated PostScript file.
    Other files used in creating 'imf2eps': gopif.h, gopimg.h
  
    Author:      Bo Nilsson, Lund Observatory    ( bo@astro.lu.se )
               
    (C) Copyright 1992, Lund Observatory
  
    Compile using
         cc -o filename filename.c


    Notice:
    -------
    I make no warranties that this program is (1) free of error, (2) con-
    sistent with any standard merchantability, or (3) meeting the require-
    ments of a particular application.
    This software shall not, partly or as a whole, participate in a process, 
    whose outcome can result in injury to a person or loss of property. It 
    is solely designed for analytical work.
    Permission to use, copy, and distribute is hereby granted without fee, 
    providing that the header above including this notice appears in all 
    copies.
                                                                  B.N.
*/

#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <time.h>
#include <stdlib.h>
#include <sys/types.h>
#include <dirent.h>

#include "gopimg.h"    /* local header       */
#include "gopfi.h"     /* local header       */

#define true 1
#define false 0
#define WI 510
#define ERROR -1
#define ok 1

char *fndfnm;

unsigned char *line, *lineR, *lineG, *lineB;   /* Dynamic line buffer */

static char b2x[] = "0123456789ABCDEF"; 


/*   This functions searches for a file from an environmental path variable
**   (recursively), and returns the file descriptor after having opened the
**   file, if successful, otherwise it returns ERROR. 
**   Inputs are the path variable, the filename without extension, and the
**   filename extension. A global pointer to a string is used.
*/

int get_fil_path(path, fname, ext)
char *path, *fname, *ext;
{
   extern char *fndfnm;
   int imfile = ERROR;
   char *remaining_path;
   DIR *dirp;

   if ((remaining_path = strchr(path, ':')) != NULL) {
      *remaining_path++ = '\0';
      if ((dirp = opendir(path)) != NULL) {
         (void)closedir(dirp);
         sprintf(fndfnm, "%s/%s%s", path, fname, ext);
	 if ((imfile = open(fndfnm, O_RDONLY, 0)) == ERROR)
            return get_fil_path(remaining_path, fname, ext);
         else
            return imfile;
      }
      else
         return get_fil_path(remaining_path, fname, ext);
   }
   else if ((dirp = opendir(path)) != NULL) {
      (void)closedir(dirp);
      sprintf(fndfnm, "%s/%s%s", path, fname, ext);
      return open(fndfnm, O_RDONLY, 0);
   }
   else 
      return ERROR;
}

 
/*  This functions searches a file from an unspecified path, filename,
**  and a filename extension, and if it finds it, opens it, and return
**  its file descriptor. With the fflags input one can determin the
**  behaviour of the function: if it shall suppress error messages or
**  not, if it shall force exit on error or continue, etc. A global
**  pointer is used (and for which is allocated memory).
**  On error, it returns ERROR.
**  It is designed to behave as an by source unknown function (with the 
**  same name) written by ContextVision AB.
*/
 
int fndif(fname, fdir, fext, fflags)
char *fname, *fdir, *fext;
int fflags;
{
   extern char *fndfnm; 
   char *ext = "                                              ";
   char *dir, *path, dpath[256];
   int i, infile = ERROR;
   DIR *dirp;
 
   if ((fndfnm = (char *)malloc(256)) == NULL) {
      if (fflags & FF_MSG)
          fprintf(stderr, "Searching image file: can't allocate memory.\n");
      if (fflags & FF_TRP)
          exit(ok);
      else
          return ERROR;
   }
 
   if (!(fflags & FF_FEX)) {
      for (i=0; *(fname + i) != '\0'; i++) {
         if (*(fname + i) == '.') {
             fext = NULL;
         }
      }
   }

   if (fext != NULL && *fext != '.')
      sprintf(ext, ".%s", fext);
   else
      ext = fext;
 
   if (fflags & FF_BNM) {
      for (i=0; *(fname + i) != '\0'; i++) {
         if (*(fname + i) == '/') {
            fname = fname + i + 1;
            i = -1;
         }
      }
      sprintf(fndfnm, "%s%s", fname, ext);
   }
   else {
      if (fdir == NULL) {
         sprintf(fndfnm, "%s%s", fname, ext);
      }
      else {
         if ((dirp = opendir(fdir)) != NULL) {
            (void)closedir(dirp);
            dir = fdir;
            sprintf(fndfnm, "%s/%s%s", dir, fname, ext);
         }
         else {
            if ((path = getenv(fdir)) != NULL) {
               strcpy(dpath, path);
	       if ((dirp = opendir(dpath)) != NULL) {
	          (void)closedir(dirp);
                  sprintf(fndfnm, "%s/%s%s", dpath, fname, ext);
               }
               else if ((infile = get_fil_path(dpath, fname, ext)) != ERROR) {
                  return infile;
               }
	       else {
		  if (fflags & FF_MSG)
		     fprintf(stderr, "Searching image file: bad path.\n");
		  if (fflags & FF_TRP) {
                     sprintf(fndfnm, "%s/%s%s", path, fname, ext);
		     exit(ok);
		  }
		  else {
                     sprintf(fndfnm, "%s/%s%s", path, fname, ext);
		     return ERROR;
		  }
	       }
	    }
            else {
	       if (fflags & FF_MSG)
		  fprintf(stderr, "Searching image file: can't read path.\n");
	       if (fflags & FF_TRP) {
                  fndfnm = fdir;
		  exit(ok);
	       }
	       else {
                  fndfnm = fdir;
		  return ERROR;
	       }
            }
         }
      }
   }
   if ((infile = open(fndfnm, O_RDONLY, 0)) == ERROR) {
      if (fflags & FF_MSG)
         fprintf(stderr,"Searching image file: can't find image file.\n");
      if (fflags & FF_TRP) {
         exit(ok);
      }
      else {
         return ERROR;
      }
   }
/* Amateurish, memory should be freed:   free((void *)fndfnm);             */

   return infile;
}
 



error(s1, s2)		/* print error message and die */
char *s1, *s2;
{
  fprintf(stderr,"Image to postscript: ");
  fprintf(stderr,s1, s2);
  fprintf(stderr,".\n");
  exit(1);
}

 
char *pixstr(hdtype)
int hdtype;
{
   char *rep = "Unknown type";

   switch (hdtype) {
     case im_b1:
                 rep = "b1 type";
                 break;
     case im_u8:
                 rep = "u8 type";
                 break;
     case im_p8:
                 rep = "p8 type";
                 break;
     case im_s8:
                 rep = "s8 type";
                 break;
     case im_c8:
                 rep = "c8 type";
                 break;
     case im_x8:
                 rep = "x8 type";
                 break;
     case im_p16:
                 rep = "p16 type";
                 break;
     case im_s16:
                 rep = "s16 type";
                 break;
     case im_c16:
                 rep = "c16 type";
                 break;
     case im_p32:
                 rep = "p32 type";
                 break;
     case im_s32:
                 rep = "s32 type";
                 break;
     case im_c32:
                 rep = "c32 type";
                 break;
     default:
                 rep = "Unknown type";
                 break;
   }
   return rep;
}


main (argc, argv)
int argc;
char *argv[];
{
  int p = 1;			/* argument-pointer */
  int infile, infileR, infileG, infileB; 
  int rgb = false, b_w = false, eps = false, text = false;
  int frame = false, rot = false;
  struct image_head hdr;
  int bpp;			/* bits per pixel */
  int buflen, nlin, ol = 0;
  register int i, j, lf = 0;    /* Scan arguments */
  char gotchar;
  long tranx, trany, scalx, scaly;
  long llx = 50, lly = 50, urx, ury, width = 0, height = 0;
  long scf, llxd, wid;
  int pxst = 16;
  char filenameR[80];
  char filenameG[80];
  char filenameB[80];
  char overline[5][80];
  time_t tnow;
  char app[L_cuserid], txtchr;
  extern char *fndfnm;
  int use_stdin = false, newpath = false, force_odd = false;
  int num_cop = 1, help_arg = false;
  char npath[256];
  char *garb, *head;

  if (argc <= 1) {
      fprintf(stderr, "Format: ");
      fprintf(stderr, "%s  [-rgb] [-bw] [-eps] [-text]\n", argv[0]);
      fprintf(stderr, "                 [-ol string] [-f] [-rot] [-p path]\n");
      fprintf(stderr, "                 [-box llx lly width height]\n");
      fprintf(stderr, "                 [-noop] [-copies NUM] [imfile]\n");
      fprintf(stderr, "                 [-help]\n");
      fprintf(stderr, "Use '-help' option for a concise information about ");
      fprintf(stderr, "the arguments.\n");
      return;
  }


  strcpy(npath, "IMGPATH");

  while ((p < argc) && (argv[p][0] == '-')) {
      if (strcmp(argv[p], "-bw") == 0) 
         b_w = true;
      else if (strcmp(argv[p], "-rgb") == 0) 
         rgb = true;
      else if (strcmp(argv[p], "-eps") == 0)
         eps = true;
      else if (strcmp(argv[p], "-box") == 0) {
         llx = 720*atoi(argv[++p])/254;
         lly = 720*atoi(argv[++p])/254;
         width = 720*atoi(argv[++p])/254;
         height = 720*atoi(argv[++p])/254;
      }
      else if (strcmp(argv[p], "-copies") == 0)
         num_cop = atoi(argv[++p]);
      else if (strcmp(argv[p], "-text") == 0) 
         text = true;
      else if (strcmp(argv[p], "-ol") == 0) 
         strcpy(overline[ol++], argv[++p]);
      else if (strcmp(argv[p], "-f") == 0)
         frame = true;
      else if (strcmp(argv[p], "-rot") == 0)
         rot = true;
      else if (strcmp(argv[p], "-p") == 0)
         strcpy(npath, argv[++p]);
      else if (strcmp(argv[p], "-odd") == 0)
         force_odd = true;
      else if (strcmp(argv[p], "-help") == 0)
         help_arg = true;
      else if (strcmp(argv[p], "-noop") == 0) 
         ;
      else error("%s illegal switch", argv[p]);
      p++;
  }

  if (help_arg) {
      fprintf(stderr, "\'-rgb\' reads three rgb u8 images\n");
      fprintf(stderr, "\'-bw\' reads a black & white u8 image\n");
      fprintf(stderr, "    \'-odd\' allows width to be odd for b & w\n");
      fprintf(stderr, "no \'-rgb\' or \'-bw\' reads a compressed color p8");
      fprintf(stderr, " image\n");
      fprintf(stderr, "\'-eps\' creates a true EPS");
      fprintf(stderr, " file without \'showpage\'\n");
      fprintf(stderr, "\'-text\' adds \'Lund Observatory\' and date\n");
      fprintf(stderr, "\'-ol\' takes \'string\' as overhead line\n");
      fprintf(stderr, "\'-f\' adds a frame, red if not -bw, black otherwise\n");
      fprintf(stderr, "\'-rot\' rotates image depending on size to fit A4\n");
      fprintf(stderr, "\'-box\' positions the image on the paper:\n");
      fprintf(stderr, "    image lower left corner coordinates in mm from");
      fprintf(stderr, " lower left corner of paper\n"); 
      fprintf(stderr, "    width and height in mm - if both are set to 0,");
      fprintf(stderr, " image size will be used\n");
      fprintf(stderr, "    - if one of them are set to 0, the image will be");
      fprintf(stderr, " scaled according to the\n");
      fprintf(stderr, "    other parameter\n");
      fprintf(stderr, "\'imfile\' is any image file in $IMGPATH :\n");
      fprintf(stderr, "    if \'-rgb\', then \'imfileR\', ");
      fprintf(stderr, "\'imfileG\', and \'imfileB\' are used\n");
      fprintf(stderr, "    if not \'-rgb\', and \'imfile\' is omitted ");
      fprintf(stderr, "or not found, \"stdin\" will be used\n");
      fprintf(stderr, "    if '-p', path is altered from IMGPATH to ");     
      fprintf(stderr, "specified path\n");
      fprintf(stderr, "\'-copies\' let you specify number of copies\n");
      fprintf(stderr, "\'-noop\' no option and no image file specified\n");
      fprintf(stderr, "\n");
      return;
  }

  if (rgb && b_w) 
      error("\'rgb\' and \'bw\' are incompatible", NULL);

  if (!rgb) {
    if (p >= argc) {
       use_stdin = true;
    }
    else if ((infile = fndif(argv[p], npath, ".imf", 0)) < 0) {

/* Notice, in this version the program tries to read from standard input if
   the specified filename and path does not exist, not only if the filename
   is omitted.                                                                */

       use_stdin = true;
    }
    if (!use_stdin) {
       if (read(infile, &hdr, sizeof(hdr)) != sizeof(hdr))
          error("error when reading header of file %s", fndfnm);
    }
    else {
       head = (char *)&hdr;
       for (j=0; j < sizeof(hdr); j++)
          *head++ = getchar();
    }
  }
  else {
     if (p >= argc)
        error("image file must be specified with option -rgb", NULL);
     strcpy(filenameR, argv[p]);
     strcpy(filenameG, argv[p]);
     strcpy(filenameB, argv[p]);
     strcat(filenameR, "R");
     strcat(filenameG, "G");
     strcat(filenameB, "B");

     infileR = fndif(filenameR, npath, ".imf", FF_MSG | FF_TRP);
     if (read(infileR, &hdr, sizeof(hdr)) != sizeof(hdr))
        error("error in reading header of %s", fndfnm);
     infileG = fndif(filenameG, npath, ".imf", FF_MSG | FF_TRP);
     if (read(infileG, &hdr, sizeof(hdr)) != sizeof(hdr))
        error("error in reading header of %s", fndfnm);
     infileB = fndif(filenameB, npath, ".imf", FF_MSG | FF_TRP);
     if (read(infileB, &hdr, sizeof(hdr)) != sizeof(hdr))
        error("error in reading header of %s", fndfnm);
  }
 
  if (!use_stdin)
     free((void *)fndfnm);         /* No more use for the filename  */

  /* Image offset  hdr.off  */
  /* file size     hdr.len  */
  /* Image height  hdr.hgh  */
  /* Image width   hdr.wid  */
  				/* Check the pixel type (hdr.typ) */
  switch (hdr.typ) {
  case im_b1:
    bpp= 1;
    break;
  case im_u8:
    bpp= 8;
    break;
  case im_p8:
    bpp= 8;
    break;
  default:
    error("pixeltype %s not supported", pixstr(hdr.typ));
    break;
  }

  printf("%%!PS-Adobe-2.0 EPSF-2.0\n");
  if (b_w)
     printf("%%%%Title: Gray PostScript File\n");
  else
     printf("%%%%Title: Color PostScript File\n");
  printf("%%%%Creator: Bo Nilsson\n");
  if (cuserid(app) == 0) {
     app[0] = '?';
     app[1] = '\0';
  }
  printf("%%%%For: \'%s\'\n", app);
  time(&tnow);
  printf("%%%%CreatingDate: %s", ctime(&tnow));
  printf("%%ImageDimension: %d x %d pixels\n", hdr.wid, hdr.hgh);
  if ((hdr.wid*hdr.hgh)%16)
     pxst = 1;
  if (!width && !height) {
     urx = hdr.wid + llx;
     ury = hdr.hgh + lly;
  }
  else if (width && !height) {
     urx = width + llx;
     ury = (hdr.hgh*width)/hdr.wid + lly;
  }
  else if (!width && height) {
     ury = height + lly;
     urx = (hdr.wid*height)/hdr.hgh + llx;
  }
  else {
     urx = width + llx;
     ury = height + lly;
  }
  if (rot && urx - llx > WI && ury - lly < WI) {
    if (frame)
       printf("%%%%BoundingBox: %d %d %d %d\n", llx-3, lly-3, ury+3, urx+3);
    else
       printf("%%%%BoundingBox: %d %d %d %d\n", llx, lly, ury, urx);
  }
  else {
    if (frame)
       printf("%%%%BoundingBox: %d %d %d %d\n", llx-3, lly-3, urx+3, ury+3);
    else
       printf("%%%%BoundingBox: %d %d %d %d\n", llx, lly, urx, ury);
  }
  printf("%%%%DocumentData: Clean7Bit\n");
  printf("%%%%LanguageLevel: 2\n");
  printf("%%%%EndComments\n");
  printf("gsave\n");
  if (num_cop > 1)
    printf("/#copies %d def\n", num_cop);
  if (!b_w)
    printf("/picstr %d string def\n", 3*pxst);
  else
    printf("/picstr %d string def\n", pxst);
  scalx = urx - llx;
  scaly = ury - lly;
  if (!rot)
    tranx = llx; 
  else
    tranx = llx + scaly;
  trany = lly;
  printf("%d %d translate\n", tranx, trany);
  if (rot) {
     if (urx - llx > WI && ury - lly < WI) {
        printf("90 rotate\n");
        wid = urx - llx;
        urx = llx + (ury - lly);
        ury = lly + wid;
     }
  }
  printf("%d %d scale\n", scalx, scaly);
  
  printf("%d %d %d ", hdr.wid, hdr.hgh, bpp);   
  printf("[%d 0 0 -%d 0 %d]\n", hdr.wid, hdr.hgh, hdr.hgh);
  printf("{currentfile\n");
  printf("picstr readhexstring pop}\n");
  if (!b_w)
    printf("false 3\n");
  if (b_w)
    nlin = hdr.hgh*(1 + (int)((((hdr.wid + 1) >> 1) << 1)/36));
  else if (rgb)
    nlin = hdr.hgh*(1 + (int)((((hdr.wid + 1) >> 1) << 1)/12));
  else
    nlin = hdr.hgh*(1 + (int)(hdr.wid/12));
  printf("%%%%BeginData: %d Hex Lines\n", nlin + 3);
  if (!b_w)
    printf("colorimage\n\n");
  else
    printf("image\n\n");


  /* Convert the image file, polar to magnitude mapping */

  if (!rgb && !b_w) {
    buflen = hdr.wid*2;
    line = (unsigned char *) malloc(buflen);
    if (!use_stdin)
       lseek(infile, hdr.off, 0);
    else {
      garb = (char *)malloc(hdr.off - sizeof(hdr));
      for (j=0; j < hdr.off - sizeof(hdr); j++)
         *garb++ = getchar();
      free((void *)garb);
    }
    for (i=0; i < hdr.hgh; i++) {
      if (!use_stdin) {
         if (read(infile, line, buflen) != buflen) 
            error("premature EOF", NULL);
      }
      else
         for (j=0; j < buflen; j++)
            line[j] = getchar();
      for (j=0; j < buflen; j += 2) {
         putchar(b2x[(line[j] >> 3) & 15]);   /*  RRRR  */
         putchar(b2x[(line[j] << 1) & 8]);    /*  R000  */
         putchar(b2x[((line[j] << 2) & 12) | ((line[j+1] >> 6) & 3)]);
         putchar(b2x[(line[j+1] >> 2) & 8]);  /*  G000  */
         putchar(b2x[(line[j+1] >> 1) & 15]); /*  BBBB  */
         putchar(b2x[(line[j+1] << 3) & 8]);  /*  B000  */
         if (++lf >= 12) {
            putchar('\n');
            lf = 0;
         }
      }
      putchar('\n');
      lf = 0;
    }
  }
  else if (rgb) { 

  /* Convert the RGB color image file */

      buflen = ((hdr.wid + 1) >> 1) << 1;
      lineR = (unsigned char *) malloc(buflen);
      lineG = (unsigned char *) malloc(buflen);
      lineB = (unsigned char *) malloc(buflen);
      lseek(infileR, hdr.off, 0);
      lseek(infileG, hdr.off, 0);
      lseek(infileB, hdr.off, 0);
      for (i=0; i < hdr.hgh; i++) {
	if (read(infileR, lineR, buflen) != buflen) 
          if (!force_odd)
	     error("premature EOF", NULL);
	if (read(infileG, lineG, buflen) != buflen)
          if (!force_odd)
	     error("premature EOF", NULL);
	if (read(infileB, lineB, buflen) != buflen)
          if (!force_odd)
	     error("premature EOF", NULL);
	for (j=0; j < hdr.wid; j++) {
	  putchar(b2x[lineR[j] >> 4]);
	  putchar(b2x[lineR[j] & 15]);
	  putchar(b2x[lineG[j] >> 4]);
	  putchar(b2x[lineG[j] & 15]);
	  putchar(b2x[lineB[j] >> 4]);
	  putchar(b2x[lineB[j] & 15]);
	  if (++lf >= 12) {
	    putchar('\n');
	    lf = 0;
	  }
	}
	putchar('\n');
	lf = 0;
      }
  }
  else {

  /* Convert the black & white image file */

      buflen = ((hdr.wid + 1) >> 1) << 1;
      line = (unsigned char *) malloc(buflen);
      if (!use_stdin)
         lseek(infile, hdr.off, 0);
      else {
	garb = (char *)malloc(hdr.off - sizeof(hdr));
	for (j=0; j < hdr.off - sizeof(hdr); j++)
	   *garb++ = getchar();
	free((void *)garb);
      }
      for (i=0; i < hdr.hgh; i++) {
	if (!use_stdin) {
	   if (read(infile, line, buflen) != buflen) 
	      if (!force_odd)
		 error("premature EOF", NULL);
        }
	else
	   for (j=0; j < buflen; j++)
	      if ((gotchar = getchar()) != EOF)
                 line[j] = (unsigned char)gotchar;
	for (j=0; j < hdr.wid; j++) {
	  putchar(b2x[line[j] >> 4]);
	  putchar(b2x[line[j] & 15]);
	  if (++lf >= 36) {
	    putchar('\n');
	    lf = 0;
	  }
	}
	putchar('\n');
	lf = 0;
      }
  }

  /* PostScript postamble */
  printf("\n%%%%EndData\n");
  printf("grestore\n");
  if (frame) {
     printf("gsave\n");
     printf("3 setlinewidth\n");
     if (!b_w)
        printf("1 0 0 setrgbcolor\n");
     printf("%d %d moveto\n", llx - 3, lly - 2);
     printf("%d %d lineto\n", urx + 2, lly - 2);
     printf("%d %d lineto\n", urx + 2, ury + 2);
     printf("%d %d lineto\n", llx - 2, ury + 2);
     printf("%d %d lineto\n", llx - 2, lly - 3);
     printf("stroke\n");
     printf("grestore\n");
  }
  if (ol) {
     printf("gsave\n");
     printf("/Helvetica findfont 18 scalefont setfont\n");
     printf("0 0 0 setrgbcolor\n");
     for (j=0; j < ol; j++) {
        printf("%d %d moveto\n(", llx, ury+5+20*(ol-j));
        printf("%s", overline[j]);
        printf(") show\n");
     }
     printf("grestore\n");
  }
  if (text) {
     printf("gsave\n");
     printf("/Times-Bold findfont 12 scalefont setfont\n");
     printf("0 0.8 0.6 setrgbcolor\n");
     printf("%d %d moveto", llx, lly-20);
     printf(" (Lund Observatory) show\n");
     printf("/Times-Roman findfont 12 scalefont setfont\n");
     printf("0 0 0 setrgbcolor\n");
     if ((urx - llx) < 250)
        printf("%d %d moveto\n(", llx, lly-35);
     else
        printf("%d %d moveto\n(", urx-130, lly-20);
     for (i=0; (txtchr = *(ctime(&tnow) + i)) != '\n' || i > 70; i++)
        putchar(txtchr);
     printf(") show\n");
     printf("grestore\n");
  }
  if (!eps)
     printf("showpage\n");
  printf("%%%%Trailer\n");
  if (!eps)
     printf("%%%%EOF\n");
  printf("\n");

  return;
}


