/***************************************************************************/
/*                                                                         */
/*                                                                         */
/* SSPHOT A Photometry Package                                             */
/* ===========================                                             */
/*                                                                         */
/* This C-program is part of the SSPHOT photometry package.                */
/* See the links below for more information and documentation.             */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package and the documentation from the web-page:    */
/*                                                                         */
/* http://www.spaennare.se/ssphot.html                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

/*************************************************************/
/*  imshift.c   SS May 95                                    */
/*                                                           */
/*************************************************************/

#include <math.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include "image.h"



void mshift(a,b,m,n,dx,dy) 
double **a;
double **b;
int m,n;
double dx,dy;
{
   int i,j;
   double f12,f34,f234,f1,f2,f3,f4,p21,pm2,e6;


   e6=1.0/6.0;

   p21=dx*dx-1.0;
   pm2=dx-2.0;

   f1=e6*(-dx)*(dx-1.0)*pm2;
   f2=0.5*p21*pm2;
   f3=0.5*(-dx)*(dx+1.0)*pm2;
   f4=e6*dx*p21;

   f12=f1+f2;
   f34=f3+f4;
   f234=f2+f34;

   for (j=1; j<=n; j++) {
      b[1][j]=f12*a[1][j]+f3*a[2][j]+f4*a[3][j];
      for (i=2; i<=(m-2); i++) {
         b[i][j]=f1*a[i-1][j]+f2*a[i][j]+f3*a[i+1][j]+f4*a[i+2][j];
      } /* for i */
      b[m-1][j]=f1*a[m-2][j]+f2*a[m-1][j]+f34*a[m][j];
      b[m][j]=f1*a[m-1][j]+f234*a[m][j];
   } /* for j */

   for (i=1; i<=m; i++) {
      for (j=1; j<=n; j++) {
         a[i][j]=b[i][j];
      } /* for j */
   } /* for i */

   p21=dy*dy-1.0;
   pm2=dy-2.0;

   f1=e6*(-dy)*(dy-1.0)*pm2;
   f2=0.5*p21*pm2;
   f3=0.5*(-dy)*(dy+1.0)*pm2;
   f4=e6*dy*p21;

   f12=f1+f2;
   f34=f3+f4;
   f234=f2+f34;
  
   for (i=1; i<=m; i++) {
      b[i][1]=f12*a[i][1]+f3*a[i][2]+f4*a[i][3];
      for (j=2; j<=(n-2); j++) {
         b[i][j]=f1*a[i][j-1]+f2*a[i][j]+f3*a[i][j+1]+f4*a[i][j+2];
      } /* for i */
      b[i][n-1]=f1*a[i][n-2]+f2*a[i][n-1]+f34*a[i][n];
      b[i][n]=f1*a[i][n-1]+f234*a[i][n];
   } /* for j */

} /* mshift */

 
int main(argc,argv) 
int argc;
char *argv[];
{
   struct image_head header; 
   short **Image;

   int w,h,w1,h1,i,j;
   double dx,dy;

   double **m1,**m2;

   double *dpm1,*dpm2;


   if (argc<5) {
      printf("Usage: %s inimage outimage dxr dyr\n",argv[0]);
      exit(0);
   } /* if */


   dx=atof(argv[3]);
   dy=atof(argv[4]);

 
   ReadHeader(argv[1],&header);

   Image = (short **)calloc(header.hgh,sizeof(short*));

   ReadImage(argv[1],Image);

   w=header.wid;
   h=header.hgh;

   w1=w+1;
   h1=h+1;


   m1=(double **)calloc(w1+1,sizeof(double*));
   dpm1=(double *)calloc((w1+1)*(h1+1),sizeof(double)); 

   m2=(double **)calloc(w1+1,sizeof(double*));
   dpm2=(double *)calloc((w1+1)*(h1+1),sizeof(double)); 

   for (i=0; i<=w1; i++) {
      m1[i]=dpm1;
      m2[i]=dpm2;
      dpm1=dpm1+h1+1;
      dpm2=dpm2+h1+1;
   } /* for i */


   for (i=0; i<w; i++) {
      for (j=0; j<h; j++) {
         m1[i+1][j+1]=(double)(Image[j][i]);
      } /* for j */
   } /* for i */

   mshift(m1,m2,w1,h1,dx,dy);


   for (i=0; i<w; i++) {
      for (j=0; j<h; j++) {
         Image[j][i]=(int)(m2[i+1][j+1]+0.5);
      } /* for j */
   } /* for i */


WriteImage(argv[2],Image,header,im_s16);

} /* End  */
