/***************************************************************************/
/*                                                                         */
/*                                                                         */
/* SSPHOT A Photometry Package                                             */
/* ===========================                                             */
/*                                                                         */
/* This C-program is part of the SSPHOT photometry package.                */
/* See the links below for more information and documentation.             */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package and the documentation from the web-page:    */
/*                                                                         */
/* http://www.spaennare.se/ssphot.html                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

/*************************************************************/
/*  mmmfw - calculates max, min, mean and FWHM of image     */
/*                                                           */
/*                              Latest update: MAR 92, SS    */
/*************************************************************/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "image.h"

double pi=3.14159265358979323;

int main(argc,argv) 
int argc;
char *argv[];
{
   struct image_head header;             
   short **image;

   int min, max, hmax, temp,i,j;
   int nrhmax;

   double tot, maxr, minr, fwhm;


   if (argc<2) {
      printf("Usage: %s image\n",argv[0]);
      exit(0);
   } /* if */

 
   ReadHeader(argv[1],&header);

   image = (short **)calloc(header.hgh,sizeof(short*));

   ReadImage(argv[1],image);

   
   max = -32768;
   min =  32767;
   tot= 0.0;

   for (i=0; i<header.hgh; i++) {
      for (j=0; j<header.wid; j++) {
         temp = image[i][j];
         tot=tot+(double)(temp); 
         if (temp > max) max = temp;
         if (temp < min) min = temp;
      } /* for j */
   } /* for i */

   hmax = max/2;
   nrhmax = 0;

   for (i=0; i<header.hgh; i++) {
      for (j=0; j<header.wid; j++) {
         temp = image[i][j];
         if (temp >= hmax) {
            nrhmax++;
         } /* if */
      } /* for j */
   } /* for i */

   fwhm = sqrt(4.0*(double)(nrhmax)/pi);
   maxr = (double)(max)/32767.0;
   minr = (double)(min)/32767.0;
   tot = tot/(double)(header.wid*header.hgh);

   printf("  max   min    max_rel    min_rel         int/A         FWHM xsize ysize\n");

   printf("%5d %5d %10.7f %10.7f %12.7f %12.7f %5d %5d\n",
           max,min,maxr,minr,tot,fwhm,header.wid,header.hgh);
    

} /* End */
