/***************************************************************************/
/*                                                                         */
/*                                                                         */
/* SSPHOT A Photometry Package                                             */
/* ===========================                                             */
/*                                                                         */
/* This C-program is part of the SSPHOT photometry package.                */
/* See the links below for more information and documentation.             */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package and the documentation from the web-page:    */
/*                                                                         */
/* http://www.spaennare.se/ssphot.html                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

/*************************************************************/
/*  mmmfwf - calculates max, min, mean and FWHM of image     */
/*                                                           */
/*                              Latest update: Dec 97, SS    */
/*************************************************************/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

double pi=3.14159265358979323;

int main(argc,argv) 
int argc;
char *argv[];
{
   
   FILE *imfile;

   double min,max,fwhm,sum,temp,hm;

   int i,j,xsize,ysize,ll,nrhm;
   
   int xys[3];

   float *d;


   if (argc<2) {
      printf("Usage: %s image.fff\n",argv[0]);
      exit(0);
   } /* if */

 
   if ((imfile = fopen(argv[1],"rb")) == NULL) {
      printf("File not found.\n");
      exit(0);
   } /* if */

   fread(&xys[1],sizeof(int),2,imfile);

   xsize=xys[1];
   ysize=xys[2];

   ll=xsize*ysize;

   d=(float *)calloc(ll+1,sizeof(float));

   fread(&d[1],sizeof(float),ll,imfile);

   
   max = -1.0e30;
   min =  1.0e30;
   sum = 0.0;

   for (i=1; i<=ll; i++) {
      temp=d[i];
      sum=sum+temp;
      if (temp > max) max=temp;
      if (temp < min) min=temp;
   } /* for i */

   hm=max/2.0;
   nrhm=0;

   for (i=1; i<=ll; i++) {
      temp=d[i];
      if (temp >= hm) {
         nrhm++;
      } /* if */
   } /* for i */

   fwhm = sqrt(4.0*(double)(nrhm)/pi);
   sum=sum/(double)(ll);

   printf("            max             min        int/A         FWHM xsize ysize\n");
   
   printf("%15.5f %15.5f %12.7f %12.7f %5d %5d\n",
           max,min,sum,fwhm,xsize,ysize);

   fclose(imfile);

} /* End  */
