/***************************************************************************/
/*                                                                         */
/*                                                                         */
/* SSPHOT A Photometry Package                                             */
/* ===========================                                             */
/*                                                                         */
/* This C-program is part of the SSPHOT photometry package.                */
/* See the links below for more information and documentation.             */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package and the documentation from the web-page:    */
/*                                                                         */
/* http://www.spaennare.se/ssphot.html                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

/*************************************************************/
/*  rebind.c   SS July 92                                    */
/*                                                           */
/*************************************************************/

#include <math.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include "image.h"


int main(argc,argv) 
int argc;
char *argv[];
{
   struct image_head header,header2; 
   short **image,**image2;
   
   int i,j,i2,j2,k,l,bf,w,h,z;

   double a,sum;

   if (argc<4) {
      printf("Usage: %s inimage outimage binfactori\n",argv[0]);
      exit(0);
   } /* if */

   bf=atoi(argv[3]);
 
   ReadHeader(argv[1],&header);

   image = (short **)calloc(header.hgh,sizeof(short*));

   ReadImage(argv[1],image);

   header2.id=image_magic;
   header2.typ=im_s16;
   header2.off=512;
   header2.hgh=header.hgh/bf;
   header2.wid=header.wid/bf;

   image2 = (short **)calloc(header2.hgh,sizeof(short*));

   CreateIm_s16(image2,header2.hgh,header2.wid);

   a=(double)(bf*bf);

   for (i=0; i<header.hgh; i=i+bf) {
      for (j=0; j<header.wid; j=j+bf) {
          i2=i/bf;
          j2=j/bf;
          sum=0.0;
          for (k=i; k<i+bf; k++) {
             for (l=j; l<j+bf; l++) {
                sum=sum+(double)(image[k][l]);
             } /* for l */
          } /* for k */
          image2[i2][j2]=(int)(sum/a);
      } /* for j */
   } /* for i */

   WriteImage(argv[2],image2,header2,im_s16);

} /* End  */
