/***************************************************************************/
/*                                                                         */
/*                                                                         */
/* SSPHOT A Photometry Package                                             */
/* ===========================                                             */
/*                                                                         */
/* This C-program is part of the SSPHOT photometry package.                */
/* See the links below for more information and documentation.             */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package and the documentation from the web-page:    */
/*                                                                         */
/* http://www.spaennare.se/ssphot.html                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

/*************************************************************/
/*  setposlf.c                                              */
/*                                                           */
/* LF by Hipparcos. Ref Johan Holmberg.                      */
/*                                                           */
/*  Latest update: Feb 96  SS, June 98                       */
/*************************************************************/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>

double pi=3.14159265358979323;

#define IM1 2147483563
#define IM2 2147483399
#define AM (1.0/IM1)
#define IMM1 (IM1-1)
#define IA1 40014
#define IA2 40692
#define IQ1 53668
#define IQ2 52774
#define IR1 12211
#define IR2 3791
#define NTAB 32
#define NDIV (1+IMM1/NTAB)
#define EPS 1.2e-7
#define RNMX (1.0-EPS)

double ran2(idum)
long *idum;
{
   int j;
   long k;
   static long idum2=123456789;
   static long iy=0; 
   static long iv[NTAB];
   double temp;

   if (*idum <= 0) {
      if (-(*idum) < 1) {
         *idum=1;
      }
      else {
         *idum=-(*idum);
      } /* if */
      idum2=(*idum);
      for (j=NTAB+7; j>=0; j--) {
         k=(*idum)/IQ1;
	 *idum=IA1*(*idum-k*IQ1)-k*IR1;
	 if (*idum < 0) {
	    *idum += IM1;
	 } /* if */
	 if (j < NTAB) {
	    iv[j]=*idum;
	 } /* if */
      } /* for j */
      iy=iv[0];
   } /* if */
   k=(*idum)/IQ1;
   *idum=IA1*(*idum-k*IQ1)-k*IR1;
   if (*idum < 0) {
      *idum += IM1;
   } /* if */
   k=idum2/IQ2;
   idum2=IA2*(idum2-k*IQ2)-k*IR2;
   if (idum2 < 0) {
      idum2 += IM2;
   } /* if */
   j=iy/NDIV;
   iy=iv[j]-idum2;
   iv[j] = *idum;
   if (iy < 1) {
      iy += IMM1;
   } /* if */
   temp=AM*(double)(iy);
   if (temp > RNMX) {
      return(RNMX);
   }
   else {
      return(temp);
   } /* if */

} /* ran2 */

int main(argc,argv) 
int argc;
char *argv[];
{

   FILE *outfile;

   double lt,rnd,dx,dy,rmin,rmin2;
   double x,y,xc,yc,inte,mag0;
   double mag,ss,intmin,intmax;

   int n1,ntot,nstep,sum,m,n,flag,flag2,sfi;
   int i,j,k,i2,xsize,ysize,nr,edge,xi,yi,seed,min,xyint;
   int ns,nf,n2,nff;

   long idum;

   double *xv,*yv;

   int f[25];

   double *ff;

   clock_t time1,timediff;


   if (argc<10) {
      printf("Usage: %s seedi xsizei ysizei xyinti(0/1) edgei rminr\n",argv[0]);
      printf("                nstarsi intmaxr intminr outfile\n");
      exit(0);
   } /* if */

  
seed=atoi(argv[1]);
xsize=atoi(argv[2]);
ysize=atoi(argv[3]);
xyint=atoi(argv[4]);
edge=atoi(argv[5]);
rmin=atof(argv[6]);
ns=atoi(argv[7]);
intmax=atof(argv[8]);
intmin=atof(argv[9]);

idum=-abs(seed)-1;

outfile=fopen(argv[10],"w");

f[0]=0;
f[1]=1; f[2]=2; f[3]=7; f[4]=13; f[5]=38; f[6]=62; f[7]=79;
f[8]=92; f[9]=77; f[10]=84; f[11]=122; f[12]=211; f[13]=335;
f[14]=452; f[15]=206; f[16]=143; f[17]=66; f[18]=60;
f[19]=60; f[20]=32; f[21]=16; f[22]=8; f[23]=5;
f[24]=1;

sum=0;

for (i=1; i<=23; i++) {
   sum=sum+f[i];
} /* for i */

nff=20;

nf=nff*sum;


ff=(double *)calloc(nf+1,sizeof(double));

sum=0;

for (i=0; i<=23; i++) {
   ss=(double)(nff)*(double)(f[i+1]);
   mag=1.0/ss;
   sum=sum+f[i];
   n1=nff*sum;
   n2=nff*(sum+f[i+1]);
   for (j=n1; j<=n2; j++) {
      ff[j]=mag*(double)(j-n1)+(double)(i);
   } /* for j*/
} /* for i */ 



xv = (double *)calloc(ns+3,sizeof(double));
yv = (double *)calloc(ns+3,sizeof(double));


time1=clock();

lt=log(10.0);

mag0=-2.5*log(intmax)/lt;

rmin2=rmin*rmin;

k=1;
j=1;

while (j <= ns) {

   n1=(int)((double)(nf)*ran2(&idum)+0.5);

   mag=ff[n1]+mag0;

   inte=exp(-0.4*lt*mag);

if (inte > intmin) {

   flag=1;

   while (flag != 0) {

      x=((double)(xsize-2*edge)*ran2(&idum)+(double)(edge));
      y=((double)(ysize-2*edge)*ran2(&idum)+(double)(edge));
      xi=(int)(x);
      yi=(int)(y);
      if (xyint == 1) {
         x=(double)(xi);
         y=(double)(yi);
      } /* if */

      i2=1;
      flag2=1;
      while ((flag2 != 0) && (i2 < k)) {
         dx=x-xv[i2];
         dy=y-yv[i2];
         if ((dx*dx+dy*dy) < rmin2) {
            flag2=0;
         } /* if */
         i2++;
      } /* while */

      if ((i2 == k) && (flag2 == 1)) {
        xv[k]=x;
        yv[k]=y;
        flag=0;
      } /* if */  

   } /* while */             
 
   k++;
   j++;

}
else {
      x=((double)(xsize-2*edge)*ran2(&idum)+(double)(edge));
      y=((double)(ysize-2*edge)*ran2(&idum)+(double)(edge));
      xi=(int)(x);
      yi=(int)(y);
      if (xyint == 1) {
         x=(double)(xi);
         y=(double)(yi);
      } /* if */
} /* if */



fprintf(outfile,"%9.3f %9.3f %13.4f\n",x,y,inte);
   

} /* while */

fclose(outfile);

timediff=clock()-time1;

} /* End */
