/***************************************************************************/
/*                                                                         */
/*                                                                         */
/* SSPHOT A Photometry Package                                             */
/* ===========================                                             */
/*                                                                         */
/* This C-program is part of the SSPHOT photometry package.                */
/* See the links below for more information and documentation.             */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package and the documentation from the web-page:    */
/*                                                                         */
/* http://www.spaennare.se/ssphot.html                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

/*************************************************************/
/*   slice.c   SS July 92                                    */
/*                                                           */
/*************************************************************/

#include <math.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include "image.h"


int main(argc,argv) 
int argc;
char *argv[];
{
   struct image_head header;    
   short **Image;
   
   int width,height,i,j;
   double x1,y1,x2,y2,qx,qy,q,le,x,y;


   if (argc<6) {
      printf("Usage: %s image x1r y1r x2r y2r\n",argv[0]);
      exit(0);
   } /* if */

 
   ReadHeader(argv[1],&header);


   Image = (short **)calloc(header.hgh,sizeof(short*));

   ReadImage(argv[1],Image);



/* Head Program */

x1=atof(argv[2]);
y1=atof(argv[3]);
x2=atof(argv[4]);
y2=atof(argv[5]);


width=header.wid;
height=header.hgh;

q=sqrt((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2));

qx=(x2-x1)/q;
qy=(y2-y1)/q;

x=x1;
y=y1;
le=0.0;

while (le <= q) {

   i=(int)(x+0.5);
   j=(int)(y+0.5);

   printf("%5.1f %8d\n",le,Image[j][i]);

   x=x+qx;
   y=y+qy;
   le=le+1.0;

} /* while */ 

WriteImage(argv[1],Image,header,im_s16);

} /* End  */
